/******************************************************************************
 *
 * Source file for harness (main) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

#define HARNESS_OPTIONS_VERSION RMIX_STRING(\
harness (harness console) %s-%s.%s\n\
Copyright (c) 2002-2006 Oak Ridge National Laboratory.\n\
See COPYING in the source distribution top-level directory or package\n\
data directory (%s) for license information.\n\
Report bugs to %s.\n)

#define HARNESS_OPTIONS_HELP "\
Usage:\n\
  harness[.debug] option\n\
Options:\n\
  -h|--help                    Print out command line help.\n\
  -v|--version                 Print out version information.\n\
  -s|--startup [HARNESS] CMD   Start up HARNESS using HARNESS_RSH environment\n\
                               variable as remote shell and CMD as command.\n\
  -sh|--startup-harnessd [HARNESS]\n\
                               Start up HARNESS using HARNESS_RSH environment\n\
                               variable as remote shell and 'harnessd -l\n\
                               libharness-console 0</dev/null 1>/dev/null\n\
                               2>/dev/null' as command.\n\
  -S|--shutdown [HARNESS]      Shut down HARNESS.\n\
  -e|--exec [HARNESS] PROGRAM  Execute PROGRAM in a child process at HARNESS.\n\
  -l|--load [HARNESS] PLUGIN   Load PLUGIN at HARNESS. Allow to load it more\n\
                               than once and do not export its symbols. Print\n\
                               out the plug-in handle.\n\
  -lu|--load-export [HARNESS] PLUGIN\n\
                               Load PLUGIN at HARNESS. Allow to load it more\n\
                               than once and export its symbols. Print out\n\
                               the plug-in handle.\n\
  -lu|--load-unique [HARNESS] PLUGIN\n\
                               Load PLUGIN at HARNESS. Allow to load it only\n\
                               once and do not export its symbols. Print out\n\
                               the plug-in handle.\n\
  -leu|--load-export-unique [HARNESS] PLUGIN\n\
                               Load PLUGIN at HARNESS. Allow to load it only\n\
                               once and export its symbols. Print out the\n\
                               plug-in handle.\n\
  -u|--unload [HARNESS] HANDLE\n\
                               Unload a plug-in at HARNESS using HANDLE.\n\
Option arguments:\n\
  HARNESS is a descriptor for a remote object reference to a harness daemon\n\
  in the form: 'PROTOCOL=<potocol> OBJECTID=<object id>' with optional name/\n\
  value pairs for 'HOST=<IP address or name>', 'PORT=<IP port>' and 'BINDING=\n\
  <protocol specific binding>'. The default is 'PROTOCOL=RPC OBJECTID=42'.\n\
  Examples are:\n\
    -  'PROTOCOL=RPC OBJECTID=42'\n\
    -  'PROTOCOL=RPC OBJECTID=42 HOST=192.168.1.1'\n\
    -  'PROTOCOL=RPC OBJECTID=42 HOST=192.168.1.1 PORT=5555'\n\
    -  'PROTOCOL=RPC OBJECTID=42 HOST=192.168.1.1 PORT=5555 BINDING=X'\n\
    -  'HOST=192.168.1.1 PORT=5555'\n\
    -  'HOST=192.168.1.1 PORT=5555 BINDING=X'\n\
Asynchronous invocations:\n\
  Appening 'a' to short options and '--async' to long options will cause a-\n\
  synchronous method invocations. For example, '--load-export-unique-async'.\n\
  Not supported by -h|--help, -v|--version and -s|--startup as they do not\n\
  invoke methods.\n\
One-way invocations:\n\
  Appening 'o' to short options and '--oneway' to long options will cause \n\
  oneway method invocations. For example, '--load-unload-oneway'. Currently\n\
  only sopported by: -s|--shutdown and -u|--unload\n\
"

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include <harness.h>

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Parses the command line options and looks up a remote object reference at a
 * RMIX harness.
 *
 * argc   = The argument vector count.
 * argv   = The argument vector (argument[0] is the program name).
 * envp   = The variable vector.
 * return = EXIT_SUCCESS on success or EXIT_FAILURE on any error.
 */
int main (int    argc,
          char **argv,
          char **envp);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/

/*
 * Parses the command line options and looks up a remote object reference at a
 * RMIX harness.
 *
 * argc   = The argument vector count.
 * argv   = The argument vector (argument[0] is the program name).
 * envp   = The variable vector.
 * return = EXIT_SUCCESS on success or EXIT_FAILURE on any error.
 */
#undef  __FUNC__
#define __FUNC__ "main"
int main (int    argc,
          char **argv,
          char **envp) {
  int                pstatus;
  char              *hparametersstr, *name, *cmd;
  unsigned int       hprovider, handle;
  rmix_remoteref_t  *harness;
  rmix_invokeref_t  *invokeref;
  rmix_parameters_t *hparameters;
  /* Check for missing arguments. */
  if (1 == argc) {
      fprintf(stderr, "harness: missing option\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
  /* Check for -h|--help option. */
  } else if ((0 == strcmp("-h",     argv[1]))||
             (0 == strcmp("--help", argv[1]))) {
    fprintf(stdout, HARNESS_OPTIONS_HELP);
    return EXIT_SUCCESS;
  /* Check for -v|--version option. */
  } else if ((0 == strcmp("-v",        argv[1]))||
             (0 == strcmp("--version", argv[1]))) {
    fprintf(stdout, HARNESS_OPTIONS_VERSION,
            RMIX_STRING(HARNESS_PACKAGE_NAME),
            RMIX_STRING(HARNESS_PACKAGE_VERSION),
            RMIX_STRING(HARNESS_PACKAGE_RELEASE),
            RMIX_STRING(HARNESS_PACKAGE_DATADIR),
            RMIX_STRING(HARNESS_PACKAGE_BUGREPORT));
    return EXIT_SUCCESS;
  /* Check for -s|--startup option. */
  } else if ((0 == strcmp("-s",        argv[1]))||
             (0 == strcmp("--startup", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = "HOST=127.0.0.1";
      cmd            = argv[2];
    } else {
      hparametersstr = argv[2];
      cmd            = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Start up harness. */
    if (0 != harnessclient_rexec3(hparametersstr, cmd)) {
      fprintf(stderr, "harness: unable to execute remote command\n");
      return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
  /* Check for -s|--startup option. */
  } else if ((0 == strcmp("-sh",                argv[1]))||
             (0 == strcmp("--startup-harnessd", argv[1]))) {
    /* Check for optional option arguments. */
    if (3 > argc) {
      hparametersstr = "HOST=127.0.0.1";
    } else {
      hparametersstr = argv[2];
    }
    /* Check for too many option arguments. */
    if (3 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Start up harness. */
#ifndef DEBUG
    if (0 != harnessclient_rexec3(hparametersstr,
        "harnessd -l libharness-console 0</dev/null 1>/dev/null 2>/dev/null")) {
#else /* DEBUG */
    if (0 != harnessclient_rexec3(hparametersstr,
  "harnessd.debug -l libharness-console 0</dev/null 1>/dev/null 2>/dev/null")) {
#endif /* DEBUG */
      fprintf(stderr, "harness: unable to execute remote command\n");
      return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
  /* Check for -S|--shutdown option. */
  } else if ((0 == strcmp("-S",         argv[1]))||
             (0 == strcmp("--shutdown", argv[1]))) {
    /* Check for optional option arguments. */
    if (3 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
    } else {
      hparametersstr = argv[2];
    }
    /* Check for too many option arguments. */
    if (3 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_RPC_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_RPC_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Shut down harness. */
    if (0 != harnessclient_kernel_shutdown_p(harness, hprovider)) {
      fprintf(stderr, "harness: unable to shutdown harness\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -Sa|--shutdown-async option. */
  } else if ((0 == strcmp("-Sa",              argv[1]))||
             (0 == strcmp("--shutdown-async", argv[1]))) {
    /* Check for optional option arguments. */
    if (3 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
    } else {
      hparametersstr = argv[2];
    }
    /* Check for too many option arguments. */
    if (3 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Shut down harness. */
    if (0 != harnessclient_kernel_shutdown_send_p(&invokeref, harness,
                                                  hprovider)) {
      fprintf(stderr, "harness: unable to shutdown harness (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Shut down harness. */
    if (0 != harnessclient_kernel_shutdown_retrieve_p(&invokeref)) {
      fprintf(stderr,
              "harness: unable to shutdown harness (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -So|--shutdown-oneway option. */
  } else if ((0 == strcmp("-So",               argv[1]))||
             (0 == strcmp("--shutdown-oneway", argv[1]))) {
    /* Check for optional option arguments. */
    if (3 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
    } else {
      hparametersstr = argv[2];
    }
    /* Check for too many option arguments. */
    if (3 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Shut down harness. */
    if (0 != harnessclient_kernel_shutdown_oneway_p(harness, hprovider)) {
      fprintf(stderr, "harness: unable to shutdown harness\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -e|--exec option. */
  } else if ((0 == strcmp("-e",     argv[1]))||
             (0 == strcmp("--exec", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_RPC_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_RPC_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Execute program. */
    if (0 != harnessclient_processes_execute_p(harness, hprovider,
                                               &handle, name, NULL)) {
      fprintf(stderr, "harness: unable to execute program\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Wait for program exit. */
    if (0 != harnessclient_processes_wait_p(harness, hprovider,
                                            handle, &pstatus)) {
      fprintf(stderr, "harness: unable to wait for program exit\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -ea|--exec-async option. */
  } else if ((0 == strcmp("-ea",          argv[1]))||
             (0 == strcmp("--exec-async", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Execute program. */
    if (0 != harnessclient_processes_execute_send_p(&invokeref, harness,
                                                    hprovider, name, NULL)) {
      fprintf(stderr, "harness: unable to execute program (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Execute program. */
    if (0 != harnessclient_processes_execute_retrieve_p(&invokeref, &handle)) {
      fprintf(stderr, "harness: unable to execute program (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Wait for program exit. */
    if (0 != harnessclient_processes_wait_send_p(&invokeref, harness,
                                                 hprovider, handle)) {
      fprintf(stderr,
              "harness: unable to wait for program exit (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Wait for program exit. */
    if (0 != harnessclient_processes_wait_retrieve_p(&invokeref, &pstatus)) {
      fprintf(stderr,
              "harness: unable to wait for program exit (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -l|--load option. */
  } else if ((0 == strcmp("-l",     argv[1]))||
             (0 == strcmp("--load", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_p(harness, hprovider, &handle,
                                          name, 0)) {
      fprintf(stderr, "harness: unable to load plug-in\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -la|--load-async option. */
  } else if ((0 == strcmp("-la",          argv[1]))||
             (0 == strcmp("--load-async", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_send_p(&invokeref, harness, hprovider,
                                               name, 0)) {
      fprintf(stderr, "harness: unable to load plug-in (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_retrieve_p(&invokeref, &handle)) {
      fprintf(stderr, "harness: unable to load plug-in (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -le|--load-export option. */
  } else if ((0 == strcmp("-le",           argv[1]))||
             (0 == strcmp("--load-export", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_p(harness, hprovider, &handle,
                                          name, HARNESS_PLUGINS_EXPORT)) {
      fprintf(stderr, "harness: unable to load plug-in\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -lea|--load-export-async option. */
  } else if ((0 == strcmp("-lea",                argv[1]))||
             (0 == strcmp("--load-export-async", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_send_p(&invokeref, harness, hprovider,
                                               name, HARNESS_PLUGINS_EXPORT)) {
      fprintf(stderr, "harness: unable to load plug-in (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_retrieve_p(&invokeref, &handle)) {
      fprintf(stderr, "harness: unable to load plug-in (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -lu|--load-unique option. */
  } else if ((0 == strcmp("-lu",           argv[1]))||
             (0 == strcmp("--load-unique", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_p(harness, hprovider, &handle,
                                          name, HARNESS_PLUGINS_UNIQUE)) {
      fprintf(stderr, "harness: unable to load plug-in\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -lua|--load-unique-async option. */
  } else if ((0 == strcmp("-lua",                argv[1]))||
             (0 == strcmp("--load-unique-async", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_send_p(&invokeref, harness, hprovider,
                                               name, HARNESS_PLUGINS_UNIQUE)) {
      fprintf(stderr, "harness: unable to load plug-in (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_retrieve_p(&invokeref, &handle)) {
      fprintf(stderr, "harness: unable to load plug-in (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -leu|--load-export-unique option. */
  } else if ((0 == strcmp("-leu",                 argv[1]))||
             (0 == strcmp("--load-export-unique", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name           = argv[2];
    } else {
      hparametersstr = argv[2];
      name           = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_p(harness, hprovider, &handle,
                                          name, HARNESS_PLUGINS_EXPORT|
                                                HARNESS_PLUGINS_UNIQUE)) {
      fprintf(stderr, "harness: unable to load plug-in\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -leua|--load-export-unique-async option. */
  } else if ((0 == strcmp("-leua",                      argv[1]))||
             (0 == strcmp("--load-export-unique-async", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      name        = argv[2];
    } else {
      hparametersstr = argv[2];
      name        = argv[3];
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_send_p(&invokeref, harness, hprovider,
                                               name, HARNESS_PLUGINS_EXPORT|
                                                     HARNESS_PLUGINS_UNIQUE)) {
      fprintf(stderr, "harness: unable to load plug-in (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Load plug-in. */
    if (0 != harnessclient_plugins_load_retrieve_p(&invokeref, &handle)) {
      fprintf(stderr, "harness: unable to load plug-in (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Print out plug-in handle. */
    fprintf(stdout, "%u\n", handle);
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -u|--unload option. */
  } else if ((0 == strcmp("-u",       argv[1]))||
             (0 == strcmp("--unload", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      handle         = atoi(argv[2]);
    } else {
      hparametersstr = argv[2];
      handle         = atoi(argv[3]);
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Unload plug-in. */
    if (0 != harnessclient_plugins_unload_p(harness, hprovider, handle)) {
      fprintf(stderr, "harness: unable to unload plug-in\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -ua|--unload-async option. */
  } else if ((0 == strcmp("-ua",            argv[1]))||
             (0 == strcmp("--unload-async", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      handle         = atoi(argv[2]);
    } else {
      hparametersstr = argv[2];
      handle         = atoi(argv[3]);
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Unload plug-in. */
    if (0 != harnessclient_plugins_unload_send_p(&invokeref, harness,
                                                 hprovider, handle)) {
      fprintf(stderr, "harness: unable to unload plug-in (send failed)\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Unload plug-in. */
    if (0 != harnessclient_plugins_unload_retrieve_p(&invokeref)) {
      fprintf(stderr, "harness: unable to unload plug-in (retrieve failed)\n");
      /* Destroy invocation reference. */
      if (0 != rmix_invokeref_destroy(&invokeref)) {
        RMIX_LOG((RMIX_WARN(unable to destroy invocation reference)))
      }
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for -uo|--unload-oneway option. */
  } else if ((0 == strcmp("-uo",             argv[1]))||
             (0 == strcmp("--unload-oneway", argv[1]))) {
    /* Check for missing option arguments. */
    if (3 > argc) {
      fprintf(stderr, "rmixregistry: missing option arguments\n");
      fprintf(stderr, "rmixregistry: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Check for optional option arguments. */
    if (4 > argc) {
      hparametersstr = HARNESS_CONSOLE_EXPORT_RPC;
      handle         = atoi(argv[2]);
    } else {
      hparametersstr = argv[2];
      handle         = atoi(argv[3]);
    }
    /* Check for too many option arguments. */
    if (4 < argc) {
      fprintf(stderr, "harness: too many option arguments\n");
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
      return EXIT_FAILURE;
    }
    /* Initialize rmix. */
    if (0 != rmix_init(RMIX_THREADS_POOL_MINIMUM, RMIX_THREADS_POOL_MAXIMUM, 0,
                       RMIX_THREADS_JOBS_MAXIMUM, RMIX_THREADS_JOBS_TIMEOUT)) {
      fprintf(stderr, "harness: unable to initialize rmix\n");
      return EXIT_FAILURE;
    }
    /* Create remote harness parameters. */
    if (0 != rmix_parameters_create4(&hparameters, hparametersstr)) {
      fprintf(stderr, "harness: unable to create remote harness parameters\n");
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default protocol parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "PROTOCOL",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_PROTOCOL))) {
      fprintf(stderr, "harness: unable to insert default protocol parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Insert default object id parameter. */
    if (0 != rmix_parameters_insert2(hparameters, "OBJECTID",
               HARNESS_STRING(HARNESS_CONSOLE_EXPORT_OBJECTID))) {
      fprintf(stderr,
              "harness: unable to insert default object id parameter\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Create remote harness reference. */
    if (0 != rmix_remoteref_create5_p2(&harness, &hprovider, hparameters)) {
      fprintf(stderr, "harness: unable to create remote harness reference\n");
      /* Destroy remote harness parameters. */
      if (0 != rmix_parameters_destroy(&hparameters)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness parameters. */
    if (0 != rmix_parameters_destroy(&hparameters)) {
      fprintf(stderr, "harness: unable to destroy remote harness parameters\n");
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness parameters)))
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Unload plug-in. */
    if (0 != harnessclient_plugins_unload_oneway_p(harness, hprovider,
                                                   handle)) {
      fprintf(stderr, "harness: unable to unload plug-in\n");
      /* Destroy remote harness reference. */
      if (0 != rmix_remoteref_destroy(&harness)) {
        RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
      }
      /* Unload harness provider. */
      if (0 != rmix_providers_unload(hprovider)) {
        RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
      }
      /* Finalize rmix. */
      if (0 != rmix_fini()) {
        RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
      }
      return EXIT_FAILURE;
    }
    /* Destroy remote harness reference. */
    if (0 != rmix_remoteref_destroy(&harness)) {
      RMIX_LOG((RMIX_WARN(unable to destroy remote harness reference)))
    }
    /* Unload harness provider. */
    if (0 != rmix_providers_unload(hprovider)) {
      RMIX_LOG((RMIX_WARN(unable to unload harness provider.)))
    }
    /* Finalize rmix. */
    if (0 != rmix_fini()) {
      RMIX_LOG((RMIX_WARN(unable to finalize rmix)))
    }
    return EXIT_SUCCESS;
  /* Check for unknown option. */
  } else {
      fprintf(stderr, "harness: unrecognized option '%s'\n", argv[1]);
      fprintf(stderr, "harness: Try `%s --help` for more information.\n",
              argv[0]);
    return EXIT_FAILURE;
  }
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
