/******************************************************************************
 *
 * Header file for the RMIX parameter (export parameter) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/* Avoid to include the content of this header file twice. */
#ifndef RMIX_PARAMETER_H
#define RMIX_PARAMETER_H

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* The export parameter data type. */
typedef struct {
  char *name;  /* export parameter name  */
  char *value; /* export parameter value */
} rmix_parameter_t;

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/*
 * Creates an export parameter.
 *
 * parameter = The export parameter to create (return).
 * name      = The export parameter name (cannot be NULL).
 * value     = The export parameter value (cannot be NULL).
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_parameter_create (rmix_parameter_t **parameter,
                           const char        *name,
                           const char        *value);

/*
 * Destroys an export parameter.
 *
 * parameter = The export parameter to destroy (also return).
 * return    = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_parameter_destroy (rmix_parameter_t **parameter);

/*
 * Duplicates an export parameter.
 *
 * parameter1 = The export parameter to create (return).
 * parameter2 = The export parameter to duplicate.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_parameter_duplicate (rmix_parameter_t       **parameter1,
                              const rmix_parameter_t  *parameter2);

/*
 * Compares two export parameters.
 *
 * diff       = The difference between both export parameters (return).
 *              diff > 0 if parameter1 < parameter2,
 *              diff < 0 if parameter1 > parameter2, and
 *              diff = 0 if parameter1 = parameter2
 * parameter1 = The export parameter to compare against.
 * parameter2 = The export parameter to compare with.
 * return     = 0 on success or -1 on any error with errno set appropriately.
 */
int rmix_parameter_compare (int                    *diff,
                            const rmix_parameter_t *parameter1,
                            const rmix_parameter_t *parameter2);

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

#endif /* RMIX_PARAMETER_H */

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
