/******************************************************************************
 *
 * Source file for rmix stream (I/O stream registry) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include "rmix.h"

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/

/*
 * Creates a data stream.
 *
 * stream   = The data stream (return).
 * incoming = The incoming stream object.
 * read     = The incoming stream read function.
 * iclose   = The incoming stream close function.
 * outgoing = The outgoing stream object.
 * write    = The outgoing stream write function.
 * oclose   = The outgoing stream close function.
 * return   = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_stream_create"
int rmix_stream_create (rmix_stream_t       **stream,
                        void                 *incoming,
                        rmix_stream_read_t    read,
                        rmix_stream_close_t   iclose,
                        void                 *outgoing,
                        rmix_stream_write_t   write,
                        rmix_stream_close_t   oclose) {
#ifdef DEBUG
  /* Check stream parameter. */
  if (NULL == stream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(stream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check incoming parameter. */
  if (NULL == incoming) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(incoming parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check read parameter. */
  if (NULL == read) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(read parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check iclose parameter. */
  if (NULL == iclose) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(iclose parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check outgoing parameter. */
  if (NULL == outgoing) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(outgoing parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check write parameter. */
  if (NULL == write) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(write parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check oclose parameter. */
  if (NULL == oclose) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(oclose parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Allocate data stream. */
  if (NULL == (*stream = malloc(sizeof(rmix_stream_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate data stream)))
    errno = errno2;
    return -1;
  }
  /* Initialize data stream. */
  (*stream)->istream.incoming = incoming;
  (*stream)->istream.read     = read;
  (*stream)->istream.close    = iclose;
  (*stream)->ostream.outgoing = outgoing;
  (*stream)->ostream.write    = write;
  (*stream)->ostream.close    = oclose;
  return 0;
}

/*
 * Destroys a data stream.
 *
 * stream = The data stream (return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_stream_destroy"
int rmix_stream_destroy (rmix_stream_t **stream) {
#ifdef DEBUG
  /* Check stream parameter. */
  if (NULL == stream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(stream parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Deallocate data stream. */
  if (NULL != *stream) {
    free(*stream);
    *stream = NULL;
  }
  return 0;
}

/*
 * Creates an incoming data stream.
 *
 * istream  = The incoming data stream (return).
 * incoming = The incoming stream object.
 * read     = The incoming stream read function.
 * close    = The incoming stream close function.
 * return   = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_istream_create"
int rmix_istream_create (rmix_istream_t     **istream,
                         void                *incoming,
                         rmix_stream_read_t   read,
                         rmix_stream_close_t  close) {
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check incoming parameter. */
  if (NULL == incoming) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(incoming parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check read parameter. */
  if (NULL == read) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(read parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check close parameter. */
  if (NULL == close) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(close parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Allocate incoming data stream. */
  if (NULL == (*istream = malloc(sizeof(rmix_istream_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate incoming data stream)))
    errno = errno2;
    return -1;
  }
  /* Initialize incoming data stream. */
  (*istream)->incoming = incoming;
  (*istream)->read     = read;
  (*istream)->close    = close;
  return 0;
}

/*
 * Destroys an incoming data stream.
 *
 * istream = The incoming data stream (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_istream_destroy"
int rmix_istream_destroy (rmix_istream_t **istream) {
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Deallocate incoming data stream. */
  if (NULL != *istream) {
    free(*istream);
    *istream = NULL;
  }
  return 0;
}

/*
 * Creates an outgoing data stream.
 *
 * ostream  = The outgoing data stream (return).
 * outgoing = The outgoing stream object.
 * write    = The outgoing stream write function.
 * close    = The outgoing stream close function.
 * return   = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_ostream_create"
int rmix_ostream_create (rmix_ostream_t      **ostream,
                         void                 *outgoing,
                         rmix_stream_write_t   write,
                         rmix_stream_close_t   close) {
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check outgoing parameter. */
  if (NULL == outgoing) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(outgoing parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check write parameter. */
  if (NULL == write) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(write parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check close parameter. */
  if (NULL == close) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(close parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Allocate outgoing data stream. */
  if (NULL == (*ostream = malloc(sizeof(rmix_ostream_t)))) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to allocate outgoing data stream)))
    errno = errno2;
    return -1;
  }
  /* Initialize outgoing data stream. */
  (*ostream)->outgoing = outgoing;
  (*ostream)->write    = write;
  (*ostream)->close    = close;
  return 0;
}

/*
 * Destroys an outgoing data stream.
 *
 * istream = The outgoing data stream (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_ostream_destroy"
int rmix_ostream_destroy (rmix_ostream_t **ostream) {
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Deallocate outgoing data stream. */
  if (NULL != *ostream) {
    free(*ostream);
    *ostream = NULL;
  }
  return 0;
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
