/******************************************************************************
 *
 * Source file for rmix valary (I/O valary registry) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include "rmix.h"

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/

/*
 * Creates a values array and all of its values. Strings and variable arrays
 * are initialized with NULL and 0-NULL respectively 
 *
 * valary = The values array to create (return).
 * valcnt = The values array size (return).
 * sigstr = The object method input/output signature.
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_valary_create"
int rmix_valary_create (void         ***valary,
                        unsigned int   *valcnt,
                        const char     *sigstr) {
  unsigned int sigidx;
#ifdef DEBUG
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if (NULL == valcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Create values. */
  for (*valary = NULL, *valcnt = 0, sigidx = 0; '\0' != sigstr[sigidx];) {
    /* Create value. */
    if (0 != rmix_value_create(valary, valcnt, sigstr, &sigidx)) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to create value)))
      /* Destroy values array. */
      if (0 != rmix_valary_destroy(valary, valcnt, sigstr)) {
        RMIX_LOG((RMIX_WARN(unable to destroy values array)))
      }
      errno = errno2;
      return -1;
    }
  }
  return 0;
}

/*
 * Destroys a values array and all of its values. The values array may be
 * incomplete, i.e. values may be missing at the end or set to NULL anywhere
 * within the values array with the exception of a variable array length.
 *
 * valary = The values array to destroy (also return).
 * valcnt = The values array size (also return).
 * sigstr = The object method input/output signature.
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_valary_destroy"
int rmix_valary_destroy (void         ***valary,
                         unsigned int   *valcnt,
                         const char     *sigstr) {
  int          result, errno2;
  unsigned int sigidx, validx;
#ifdef DEBUG
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if (NULL == valcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valary parameter. */
  if ((NULL == *valary)&&(0 != *valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valcnt parameter. */
  if ((0 == *valcnt)&&(NULL != *valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(signature parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  result = 0;
  errno2 = 0;
  /* Destroy values. */
  for (sigidx = 0, validx = 0; validx < *valcnt;) {
    /* Destroy value. */
    if (0 != rmix_value_destroy(*valary, &validx, sigstr, &sigidx)) {
      errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to destroy value)))
      result = -1;
    }
  }
  /* Deallocate values array. */
  if (NULL != *valary) {
    free(*valary);
    *valary = NULL;
  }
  *valcnt = 0;
  if (0 != errno2) {
    errno = errno2;
  }
  return result;
}

/*
 * Destroys all variable length values of a values array. The values array may
 * be incomplete, i.e. values may be missing at the end or set to NULL anywhere
 * within the values array with the exception of a variable array length.
 *
 * valary = The values array to destroy (also return).
 * valcnt = The values array size (also return).
 * sigstr = The object method input/output signature.
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_valary_destroy2"
int rmix_valary_destroy2 (void         **valary,
                          unsigned int   valcnt,
                          const char    *sigstr) {
  int          result, errno2;
  unsigned int sigidx, validx;
#ifdef DEBUG
  /* Check valary parameter. */
  if ((NULL == valary)&&(0 != valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if ((0 == valcnt)&&(NULL != valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(signature parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  result = 0;
  errno2 = 0;
  /* Destroy variable length values. */
  for (sigidx = 0, validx = 0; validx < valcnt;) {
    /* Destroy value. */
    if (0 != rmix_value_destroy2(valary, &validx, sigstr, &sigidx)) {
      errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to destroy variable length value)))
      result = -1;
    }
  }
  if (0 != errno2) {
    errno = errno2;
  }
  return result;
}

/*
 * Destroys a values array and all of its values. The values array may be
 * incomplete, i.e. values may be missing at the end or set to NULL anywhere
 * within the values array with the exception of a variable array length.
 * (Pthread cleanup function.)
 *
 * cleanup = The values array cleanup data to cleanup.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_valary_cleanup"
void rmix_valary_cleanup (void *cleanup) {
#ifdef DEBUG
  /* Check cleanup parameter. */
  if (NULL == cleanup) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cleanup parameter is null)))
    errno = EINVAL;
    return;
  }
#endif /* DEBUG */
  /* Destroy values array. */
  if (0 != rmix_valary_destroy(&((rmix_valary_cleanup_t*)cleanup)->valary,
                               &((rmix_valary_cleanup_t*)cleanup)->valcnt,
                                ((rmix_valary_cleanup_t*)cleanup)->sigstr)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to destroy values array)))
    errno = errno2;
  }
}

/*
 * Destroys all variable length values of a values array. The values array may
 * be incomplete, i.e. values may be missing at the end or set to NULL anywhere
 * within the values array with the exception of a variable array length.
 * (Pthread cleanup function.)
 *
 * cleanup = The values array cleanup data to cleanup.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_valary_cleanup2"
void rmix_valary_cleanup2 (void *cleanup) {
#ifdef DEBUG
  /* Check cleanup parameter. */
  if (NULL == cleanup) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cleanup parameter is null)))
    errno = EINVAL;
    return;
  }
#endif /* DEBUG */
  /* Destroy values. */
  if (0 != rmix_valary_destroy2(((rmix_valary_cleanup_t*)cleanup)->valary,
                                ((rmix_valary_cleanup_t*)cleanup)->valcnt,
                                ((rmix_valary_cleanup_t*)cleanup)->sigstr)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to destroy values)))
    errno = errno2;
  }
}

/*
 * Moves all values from one values array to another. Both values arrays must
 * already exist and must be equal with the exception of strings and variable
 * arrays.
 *
 * valary1 = The values array to move values into.
 * valary1 = The values array to move values from.
 * valcnt  = The values array size.
 * sigstr  = The object method input/output signature.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_valary_move"
int rmix_valary_move (void               **valary1,
                      void               **valary2,
                      const unsigned int   valcnt,
                      const char          *sigstr) {
  unsigned int sigidx, validx;
#ifdef DEBUG
  /* Check valary1 parameter. */
  if ((NULL == valary1)&&(0 != valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary1 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary2 parameter. */
  if ((NULL == valary2)&&(0 != valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if ((0 == valcnt)&&(NULL != valary1)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if ((0 == valcnt)&&(NULL != valary2)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(signature parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Move values. */
  for (sigidx = 0, validx = 0; validx < valcnt;) {
    /* Move value. */
    if (0 != rmix_value_move(valary1, valary2, &validx, sigstr, &sigidx)) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to move value)))
      errno = errno2;
      return -1;
    }
  }
  return 0;
}

/*
 * Creates a value based on the type signature, reallocates the values array,
 * stores the allocated value in it, and advances the signature index. Strings
 * and variable arrays are initialized with NULL and 0-NULL respectively 
 *
 * valary = The values array (also return).
 * valcnt = The values array size (also return).
 * sigstr = The type signature.
 * sigidx = The type signature index (also return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_value_create"
int rmix_value_create (void         ***valary,
                       unsigned int   *valcnt,
                       const char     *sigstr,
                       unsigned int   *sigidx) {
#ifdef DEBUG
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if (NULL == valcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valary parameter. */
  if ((NULL == *valary)&&(0 != *valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valcnt parameter. */
  if ((0 == *valcnt)&&(NULL != *valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigidx parameter. */
  if (NULL == sigidx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigidx parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for type in signature. */
  switch (sigstr[*sigidx]) {
    /* Check for end of signature. */
    case '\0': {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(unexpected end of signature)))
      errno = EINVAL;
      return -1;
    }
    /* Check for char type. */
    case RMIX_SIGNATURE_CTYPE(char):
    case RMIX_SIGNATURE_CTYPE(signed_char):
    case RMIX_SIGNATURE_CTYPE(unsigned_char): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate char value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(char))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate char value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for short type. */
    case RMIX_SIGNATURE_CTYPE(signed_short):
    case RMIX_SIGNATURE_CTYPE(unsigned_short): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate short value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(short))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate short value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for int type. */
    case RMIX_SIGNATURE_CTYPE(signed_int):
    case RMIX_SIGNATURE_CTYPE(unsigned_int): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate int value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(int))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate int value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate long value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(long))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate long value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for long long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate long long value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(long long))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate long long value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for float type. */
    case RMIX_SIGNATURE_CTYPE(float): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate float value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(float))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate float value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for double type. */
    case RMIX_SIGNATURE_CTYPE(double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate double value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(double))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate double value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for long double type. */
    case RMIX_SIGNATURE_CTYPE(long_double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate long double value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(long double))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate long double value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for string type. */
    case RMIX_SIGNATURE_CTYPE(string): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate string value. */
      if (0 != rmix_value_malloc(valary, valcnt, 0)) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate string value)))
        errno = errno2;
        return -1;
      }
      break;
    }
    /* Check for variable array type. */
    case RMIX_SIGNATURE_CTYPE(variable_array_): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Allocate variable array length value. */
      if (0 != rmix_value_malloc(valary, valcnt, sizeof(int))) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate variable array length value)))
        errno = errno2;
        return -1;
      }
      /* Initialize variable array length value. */
      *(unsigned int*)(*valary)[(*valcnt) - 1] = 0;
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate char array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate char array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate short array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate short value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate int array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate int array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate long array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate long array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate long long array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate long long array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate float array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate float array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate double array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate double array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate long double array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate long double array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate string array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate string array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          void         **valary2;
          unsigned int   valcnt2, sigidx2;
          /* Initialize array/structure array. */
          valary2 = NULL;
          valcnt2 = 0;
          /* Set array/structure array signature index. */
          sigidx2 = *sigidx;
          /* Create array/structure array member. */
          if (0 != rmix_value_create(&valary2, &valcnt2, sigstr, &sigidx2)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to create array/structure array member)))
            /* Destroy array/structure array members. */
            if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                         &sigstr[*sigidx])) {
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array members)))
            }
            errno = errno2;
            return -1;
          }
          /* Destroy array/structure array member. */
          if (0 != rmix_valary_destroy(&valary2, &valcnt2, &sigstr[*sigidx])) {
            int errno2 = errno;
            RMIX_LOG((
              RMIX_WARN(unable to destroy array/structure array members)))
            errno = errno2;
            return -1;
          }
          /* Allocate array/structure array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((
              RMIX_WARN(unable to allocate  array/structure array value)))
            errno = errno2;
            return -1;
          }
          /* Advance signature index. */
          *sigidx = sigidx2;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for fixed array type. */
    case RMIX_SIGNATURE_CTYPE(fixed_array_): {
      unsigned int length, sigidx2;
      /* Save signature index. */
      sigidx2 = *sigidx;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate char array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(char))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate char array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate short array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(short))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate short value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate int array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(int))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate int array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate long array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(long))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate long array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate long long array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(long long))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate long long array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate float array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(float))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate float array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate double array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(double))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate double array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate long double array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(long double))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate long double array value)))
            errno = errno2;
            return -1;
          }
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Allocate string array value. */
          if (0 != rmix_value_malloc(valary, valcnt,
                                     length * sizeof(char*))) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to allocate string array value)))
            errno = errno2;
            return -1;
          }
          /* Initialize string array members. */
          memset((*valary)[(*valcnt) - 1], 0, length * sizeof(char*));
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          void         **valary2;
          unsigned int   valcnt2, validx2, sigidx3;
          /* Create array/structure array members. */
          for (valary2 = NULL, valcnt2 = 0,
               validx2 = 0; validx2 < length; validx2++) {
            /* Set array/structure array signature index. */
            sigidx3 = *sigidx;
            /* Create array/structure array member. */
            if (0 != rmix_value_create(&valary2, &valcnt2,
                                       sigstr, &sigidx3)) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to create array/structure array member)))
              /* Destroy array/structure array members. */
              if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                           &sigstr[sigidx2])) {
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array members)))
              }
              errno = errno2;
              return -1;
            }
          }
          /* Allocate array/structure array value. */
          if (0 != rmix_value_malloc(valary, valcnt, 0)) {
            int errno2 = errno;
            RMIX_LOG((
              RMIX_WARN(unable to allocate  array/structure array value)))
            /* Destroy array/structure array members. */
            if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                         &sigstr[sigidx2])) {
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array members)))
            }
            errno = errno2;
            return -1;
          }
          /* Initialize array/structure array value. */
          (*valary)[(*valcnt) - 1] = valary2;
          /* Advance signature index. */
          *sigidx = sigidx2;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for structure type. */
    case RMIX_SIGNATURE_CTYPE(structure_): {
      void         **valary2;
      unsigned int   length, valcnt2, validx2, sigidx2;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Create structure members. */
      for (valary2 = NULL, valcnt2 = 0, sigidx2 = *sigidx,
           validx2 = 0; validx2 < length; validx2++) {
        /* Create structure member. */
        if (0 != rmix_value_create(&valary2, &valcnt2, sigstr, sigidx)) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to create structure member)))
          /* Destroy structure members. */
          if (0 != rmix_valary_destroy(&valary2, &valcnt2, &sigstr[sigidx2])) {
            RMIX_LOG((RMIX_WARN(unable to destroy structure members)))
          }
          errno = errno2;
          return -1;
        }
      }
      /* Allocate structure value. */
      if (0 != rmix_value_malloc(valary, valcnt, 0)) {
        int errno2 = errno;
        RMIX_LOG((RMIX_WARN(unable to allocate structure value)))
        /* Destroy structure members. */
        if (0 != rmix_valary_destroy(&valary2, &valcnt2, &sigstr[sigidx2])) {
          RMIX_LOG((RMIX_WARN(unable to destroy structure members)))
        }
        errno = errno2;
        return -1;
      }
      /* Initialize array/structure array value. */
      (*valary)[(*valcnt) - 1] = valary2;
      break;
    }
    /* Catch invalid type in signature. */
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(invalid type in signature)))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Deallocates a value and advances the values array and signature indices.
 *
 * valary = The values array.
 * validx = The values array index (also return).
 * sigstr = The type signature.
 * sigidx = The type signature index (also return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_value_destroy"
int rmix_value_destroy (void         **valary,
                        unsigned int  *validx,
                        const char    *sigstr,
                        unsigned int  *sigidx) {
#ifdef DEBUG
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check validx parameter. */
  if (NULL == validx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(validx parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigidx parameter. */
  if (NULL == sigidx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigidx parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for type in signature. */
  switch (sigstr[*sigidx]) {
    /* Check for end of signature. */
    case '\0': {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(unexpected end of signature)))
      errno = EINVAL;
      return -1;
    }
    /* Check for integer, floating point and string types. */
    case RMIX_SIGNATURE_CTYPE(char):
    case RMIX_SIGNATURE_CTYPE(signed_char):
    case RMIX_SIGNATURE_CTYPE(unsigned_char):
    case RMIX_SIGNATURE_CTYPE(signed_short):
    case RMIX_SIGNATURE_CTYPE(unsigned_short):
    case RMIX_SIGNATURE_CTYPE(signed_int):
    case RMIX_SIGNATURE_CTYPE(unsigned_int):
    case RMIX_SIGNATURE_CTYPE(signed_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long):
    case RMIX_SIGNATURE_CTYPE(signed_long_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
    case RMIX_SIGNATURE_CTYPE(float):
    case RMIX_SIGNATURE_CTYPE(double):
    case RMIX_SIGNATURE_CTYPE(long_double):
    case RMIX_SIGNATURE_CTYPE(string): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Deallocate value. */
      if (NULL != valary[*validx]) {
        free(valary[*validx]);
        valary[*validx] = NULL;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for variable array type. */
    case RMIX_SIGNATURE_CTYPE(variable_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Save array length value. */
      length = *(unsigned int*)valary[*validx];
      /* Deallocate array length value. */
      free(valary[*validx]);
      valary[*validx] = NULL;
      /* Advance value index. */
      (*validx)++;
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for integer, floating point and string types. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char):
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short):
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int):
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long):
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
        case RMIX_SIGNATURE_CTYPE(float):
        case RMIX_SIGNATURE_CTYPE(double):
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate string array members. */
          for (index = 0; index < length; index++) {
            /* Deallocate string array member. */
            if (NULL != ((char**)valary[*validx])[index]) {
              free(((char**)valary[*validx])[index]);
            }
          }
          /* Deallocate string array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          /* Check for zero array/structure array length. */
          if (0 == length) {
            void         **valary2;
            unsigned int   valcnt2, sigidx2;
            /* Initialize array/structure array. */
            valary2 = NULL;
            valcnt2 = 0;
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Create array/structure array member. */
            if (0 != rmix_value_create(&valary2, &valcnt2, sigstr, &sigidx2)) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to create array/structure array member)))
              /* Destroy array/structure array members. */
              if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                           &sigstr[*sigidx])) {
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array member)))
              }
              errno = errno2;
              return -1;
            }
            /* Destroy array/structure array member. */
            if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                         &sigstr[*sigidx])) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
              errno = errno2;
              return -1;
            }
            /* Advance value index. */
            (*validx)++;
            return 0;
          } else {
            int          errno2, result;
            unsigned int validx2, index, sigidx2;
            result = 0;
            errno2 = 0;
            /* Destroy array/structure array members. */
            for (validx2 = 0, index = 0; index < length; index++) {
              /* Set array/structure array signature index. */
              sigidx2 = *sigidx;
              /* Destroy array/structure array member. */
              if (0 != rmix_value_destroy((void**)valary[*validx], &validx2,
                                          sigstr, &sigidx2)) {
                errno2 = errno;
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array member)))
                result = -1;
              }
            }
            /* Deallocate array/structure array value. */
            if (NULL != valary[*validx]) {
              free(valary[*validx]);
              valary[*validx] = NULL;
            }
            /* Advance signature index. */
            *sigidx = sigidx2;
            /* Advance value index. */
            (*validx)++;
            if (0 != errno2) {
              errno = errno2;
            }
            return result;
          }
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for fixed array type. */
    case RMIX_SIGNATURE_CTYPE(fixed_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for integer, floating point and string types. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char):
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short):
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int):
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long):
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
        case RMIX_SIGNATURE_CTYPE(float):
        case RMIX_SIGNATURE_CTYPE(double):
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate string array members. */
          for (index = 0; index < length; index++) {
            /* Deallocate string array member. */
            if (NULL != ((char**)valary[*validx])[index]) {
              free(((char**)valary[*validx])[index]);
            }
          }
          /* Deallocate string array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          int          errno2, result;
          unsigned int validx2, index, sigidx2;
          result = 0;
          errno2 = 0;
          /* Destroy array/structure array members. */
          for (validx2 = 0, index = 0; index < length; index++) {
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Destroy array/structure array member. */
            if (0 != rmix_value_destroy((void**)valary[*validx], &validx2,
                                        sigstr, &sigidx2)) {
              errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
              result = -1;
            }
          }
          /* Deallocate array/structure array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance signature index. */
          *sigidx = sigidx2;
          /* Advance value index. */
          (*validx)++;
          if (0 != errno2) {
            errno = errno2;
          }
          return result;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for structure type. */
    case RMIX_SIGNATURE_CTYPE(structure_): {
      int            result, errno2;
      unsigned int   length, validx2, index;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      result = 0;
      errno2 = 0;
      /* Destroy structure members. */
      for (validx2 = 0, index = 0; index < length; index++) {
        /* Destroy structure value. */
        if (0 != rmix_value_destroy((void**)valary[*validx], &validx2,
                                    sigstr, sigidx)) {
          errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to destroy structure member)))
          result = -1;
        }
      }
      /* Deallocate structure value. */
      if (NULL != valary[*validx]) {
        free(valary[*validx]);
        valary[*validx] = NULL;
      }
      /* Advance value index. */
      (*validx)++;
      if (0 != errno2) {
        errno = errno2;
      }
      return result;
    }
    /* Catch invalid type in signature. */
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(invalid type in signature)))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Deallocates a value if it is a string or variable array and advances the
 * values array and signature indices.
 *
 * valary = The values array.
 * validx = The values array index (also return).
 * sigstr = The type signature.
 * sigidx = The type signature index (also return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_value_destroy2"
int rmix_value_destroy2 (void         **valary,
                         unsigned int  *validx,
                         const char    *sigstr,
                         unsigned int  *sigidx) {
#ifdef DEBUG
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check validx parameter. */
  if (NULL == validx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(validx parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigidx parameter. */
  if (NULL == sigidx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigidx parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for type in signature. */
  switch (sigstr[*sigidx]) {
    /* Check for end of signature. */
    case '\0': {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(unexpected end of signature)))
      errno = EINVAL;
      return -1;
    }
    /* Check for integer, floating point and string types. */
    case RMIX_SIGNATURE_CTYPE(char):
    case RMIX_SIGNATURE_CTYPE(signed_char):
    case RMIX_SIGNATURE_CTYPE(unsigned_char):
    case RMIX_SIGNATURE_CTYPE(signed_short):
    case RMIX_SIGNATURE_CTYPE(unsigned_short):
    case RMIX_SIGNATURE_CTYPE(signed_int):
    case RMIX_SIGNATURE_CTYPE(unsigned_int):
    case RMIX_SIGNATURE_CTYPE(signed_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long):
    case RMIX_SIGNATURE_CTYPE(signed_long_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
    case RMIX_SIGNATURE_CTYPE(float):
    case RMIX_SIGNATURE_CTYPE(double):
    case RMIX_SIGNATURE_CTYPE(long_double):
    case RMIX_SIGNATURE_CTYPE(string): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for variable array type. */
    case RMIX_SIGNATURE_CTYPE(variable_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Save array length value. */
      length = *(unsigned int*)valary[*validx];
      /* Advance value index. */
      (*validx)++;
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for integer, floating point and string types. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char):
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short):
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int):
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long):
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
        case RMIX_SIGNATURE_CTYPE(float):
        case RMIX_SIGNATURE_CTYPE(double):
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate string array members. */
          for (index = 0; index < length; index++) {
            /* Deallocate string array member. */
            if (NULL != ((char**)valary[*validx])[index]) {
              free(((char**)valary[*validx])[index]);
            }
          }
          /* Deallocate string array value. */
          if (NULL != valary[*validx]) {
            free(valary[*validx]);
            valary[*validx] = NULL;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          /* Check for zero array/structure array length. */
          if (0 == length) {
            void         **valary2;
            unsigned int   valcnt2, sigidx2;
            /* Initialize array/structure array. */
            valary2 = NULL;
            valcnt2 = 0;
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Create array/structure array member. */
            if (0 != rmix_value_create(&valary2, &valcnt2, sigstr, &sigidx2)) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to create array/structure array member)))
              /* Destroy array/structure array members. */
              if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                           &sigstr[*sigidx])) {
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array member)))
              }
              errno = errno2;
              return -1;
            }
            /* Destroy array/structure array member. */
            if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                         &sigstr[*sigidx])) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
              errno = errno2;
              return -1;
            }
            /* Advance value index. */
            (*validx)++;
            return 0;
          } else {
            int          errno2, result;
            unsigned int validx2, index, sigidx2;
            result = 0;
            errno2 = 0;
            /* Destroy array/structure array members. */
            for (validx2 = 0, index = 0; index < length; index++) {
              /* Set array/structure array signature index. */
              sigidx2 = *sigidx;
              /* Destroy array/structure array member. */
              if (0 != rmix_value_destroy2((void**)valary[*validx], &validx2,
                                           sigstr, &sigidx2)) {
                errno2 = errno;
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array member)))
                result = -1;
              }
            }
            /* Deallocate array/structure array value. */
            if (NULL != valary[*validx]) {
              free(valary[*validx]);
              valary[*validx] = NULL;
            }
            /* Advance signature index. */
            *sigidx = sigidx2;
            /* Advance value index. */
            (*validx)++;
            if (0 != errno2) {
              errno = errno2;
            }
            return result;
          }
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for fixed array type. */
    case RMIX_SIGNATURE_CTYPE(fixed_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for integer, floating point and string types. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char):
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short):
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int):
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long):
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
        case RMIX_SIGNATURE_CTYPE(float):
        case RMIX_SIGNATURE_CTYPE(double):
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Advance value index. */
          (*validx)++;
          break;
        }
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Deallocate string array members. */
          for (index = 0; index < length; index++) {
            /* Deallocate string array member. */
            if (NULL != ((char**)valary[*validx])[index]) {
              free(((char**)valary[*validx])[index]);
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          int          errno2, result;
          unsigned int validx2, index, sigidx2;
          result = 0;
          errno2 = 0;
          /* Destroy array/structure array members. */
          for (validx2 = 0, index = 0; index < length; index++) {
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Destroy array/structure array member. */
            if (0 != rmix_value_destroy2((void**)valary[*validx], &validx2,
                                         sigstr, &sigidx2)) {
              errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
              result = -1;
            }
          }
          /* Advance signature index. */
          *sigidx = sigidx2;
          /* Advance value index. */
          (*validx)++;
          if (0 != errno2) {
            errno = errno2;
          }
          return result;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for structure type. */
    case RMIX_SIGNATURE_CTYPE(structure_): {
      int            result, errno2;
      unsigned int   length, validx2, index;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      result = 0;
      errno2 = 0;
      /* Destroy structure members. */
      for (validx2 = 0, index = 0; index < length; index++) {
        /* Destroy structure value. */
        if (0 != rmix_value_destroy2((void**)valary[*validx], &validx2,
                                     sigstr, sigidx)) {
          errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to destroy structure member)))
          result = -1;
        }
      }
      /* Advance value index. */
      (*validx)++;
      if (0 != errno2) {
        errno = errno2;
      }
      return result;
    }
    /* Catch invalid type in signature. */
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(invalid type in signature)))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Moves a value and advances the values array and signature indices.
 *
 * valary1 = The values array to move the value into.
 * valary2 = The values array to move the value from.
 * validx  = The values array index (also return).
 * sigstr  = The type signature.
 * sigidx  = The type signature index (also return).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_value_move"
int rmix_value_move (void         **valary1,
                     void         **valary2,
                     unsigned int  *validx,
                     const char    *sigstr,
                     unsigned int  *sigidx) {
#ifdef DEBUG
  /* Check valary1 parameter. */
  if (NULL == valary1) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary1 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary2 parameter. */
  if (NULL == valary2) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary2 parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check validx parameter. */
  if (NULL == validx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(validx parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigidx parameter. */
  if (NULL == sigidx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigidx parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for type in signature. */
  switch (sigstr[*sigidx]) {
    /* Check for end of signature. */
    case '\0': {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(unexpected end of signature)))
      errno = EINVAL;
      return -1;
    }
    /* Check for char type. */
    case RMIX_SIGNATURE_CTYPE(char):
    case RMIX_SIGNATURE_CTYPE(signed_char):
    case RMIX_SIGNATURE_CTYPE(unsigned_char): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy char value. */
      *(char*)valary1[*validx] = *(char*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for short type. */
    case RMIX_SIGNATURE_CTYPE(signed_short):
    case RMIX_SIGNATURE_CTYPE(unsigned_short): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy short value. */
      *(short*)valary1[*validx] = *(short*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for int type. */
    case RMIX_SIGNATURE_CTYPE(signed_int):
    case RMIX_SIGNATURE_CTYPE(unsigned_int): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy int value. */
      *(int*)valary1[*validx] = *(int*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy long value. */
      *(long*)valary1[*validx] = *(long*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for long long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy long long value. */
      *(long long*)valary1[*validx] = *(long long*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for float type. */
    case RMIX_SIGNATURE_CTYPE(float): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy float value. */
      *(float*)valary1[*validx] = *(float*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for double type. */
    case RMIX_SIGNATURE_CTYPE(double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy double value. */
      *(double*)valary1[*validx] = *(double*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for long double type. */
    case RMIX_SIGNATURE_CTYPE(long_double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Copy long double value. */
      *(long double*)valary1[*validx] = *(long double*)valary2[*validx];
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for string type. */
    case RMIX_SIGNATURE_CTYPE(string): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Move string value. */
      valary1[*validx] = valary2[*validx];
      valary2[*validx] = NULL;
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Check for variable array type. */
    case RMIX_SIGNATURE_CTYPE(variable_array_): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Advance values array index. */
      (*validx)++;
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for other types than array array and array structure. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char):
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short):
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int):
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long):
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long):
        case RMIX_SIGNATURE_CTYPE(long_double):
        case RMIX_SIGNATURE_CTYPE(string): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Move variable array value. */
          valary1[*validx] = valary2[*validx];
          valary2[*validx] = NULL;
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          void         **valary3;
          unsigned int   valcnt3, sigidx3;
          /* Initialize array/structure array. */
          valary3 = NULL;
          valcnt3 = 0;
          /* Set array/structure array signature index. */
          sigidx3 = *sigidx;
          /* Create array/structure array member. */
          if (0 != rmix_value_create(&valary3, &valcnt3, sigstr, &sigidx3)) {
            int errno2 = errno;
            RMIX_LOG((RMIX_WARN(unable to create array/structure array member)))
            /* Destroy array/structure array members. */
            if (0 != rmix_valary_destroy(&valary3, &valcnt3,
                                         &sigstr[*sigidx])) {
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
            }
            errno = errno2;
            return -1;
          }
          /* Destroy array/structure array member. */
          if (0 != rmix_valary_destroy(&valary3, &valcnt3, &sigstr[*sigidx])) {
            int errno2 = errno;
            RMIX_LOG((
              RMIX_WARN(unable to destroy array/structure array member)))
            errno = errno2;
            return -1;
          }
          /* Move array/structure array value. */
          valary1[*validx] = valary2[*validx];
          valary2[*validx] = NULL;
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      /* Move variable array length value. */
      *(int*)valary1[(*validx) - 2] = *(int*)valary2[(*validx) - 2];
      *(int*)valary2[(*validx) - 2] = 0;
      break;
    }
    /* Check for fixed array type. */
    case RMIX_SIGNATURE_CTYPE(fixed_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return -1;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy char array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(char));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy short array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(short));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy int array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(int));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy long array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(long));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy long long array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(long long));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy float array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(float));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy double array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(double));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Copy long double array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(long double));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Move string array value. */
          memcpy(valary1[*validx], valary2[*validx],
                 length * sizeof(char*));
          memset(valary2[*validx], 0, length * sizeof(char*));
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          unsigned int validx3, index, sigidx3;
          /* Move array/structure array members. */
          for (validx3 = 0, index = 0; index < length; index++) {
            /* Set array/structure array signature index. */
            sigidx3 = *sigidx;
            /* Move array/structure array member. */
            if (0 != rmix_value_move((void**)valary1[*validx],
                                     (void**)valary2[*validx],
                                     &validx3, sigstr, &sigidx3)) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to move array/structure array member)))
              errno = errno2;
              return -1;;
            }
          }
          /* Advance signature index. */
          *sigidx = sigidx3;
          /* Advance values array index. */
          (*validx)++;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for structure type. */
    case RMIX_SIGNATURE_CTYPE(structure_): {
      unsigned int length, validx3, index;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Move structure array members. */
      for (validx3 = 0, index = 0; index < length; index++) {
        /* Move array/structure array member. */
        if (0 != rmix_value_move((void**)valary1[*validx],
                                 (void**)valary2[*validx],
                                 &validx3, sigstr, sigidx)) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to move array/structure array member)))
          errno = errno2;
          return -1;;
        }
      }
      /* Advance values array index. */
      (*validx)++;
      break;
    }
    /* Catch invalid type in signature. */
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(invalid type in signature)))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Allocates a value, reallocates the values array and stores the allocated
 * value in it.
 *
 * valary = The values array (also return).
 * valcnt = The values array size (also return).
 * length = The value length in bytes (may be 0).
 * return = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_value_malloc"
int rmix_value_malloc (void         ***valary,
                       unsigned int   *valcnt,
                       unsigned int    length) {
  void *value, **valary2;
#ifdef DEBUG
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if (NULL == valcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valary parameter. */
  if ((NULL == *valary)&&(0 != *valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valcnt parameter. */
  if ((0 == *valcnt)&&(NULL != *valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for value size of 0. */
  if (0 == length) {
    /* Set value to NULL. */
    value = NULL;
  } else {
    /* Allocate value. */
    if (NULL == (value = malloc(length))) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to allocate value)))
      errno = errno2;
      return -1;
    }
  }
  /* Increase values count. */
  (*valcnt)++;
#if 1 < RMIX_VALARY_BLOCK
  /* Check if next block needs to be allocated. */
  if (1 == (*valcnt) % RMIX_VALARY_BLOCK) {
#endif /* 1 < RMIX_VALARY_BLOCK */
    /* Reallocate values array. */
    if (NULL == (valary2 = realloc(*valary,
                                   ((*valcnt) + RMIX_VALARY_BLOCK - 1) *
                                   sizeof(void*)))) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to reallocate values array)))
      /* Decrease values count. */
      (*valcnt)--;
      /* Deallocate value. */
      if (NULL != value) {
        free(value);
      }
      errno = errno2;
      return -1;
    }
    *valary = valary2;
#if 1 < RMIX_VALARY_BLOCK
  }
#endif /* 1 < RMIX_VALARY_BLOCK */
  /* Store value in values array. */
  (*valary)[(*valcnt) - 1] = value;
  return 0;
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
