/******************************************************************************
 *
 * Source file for rmix-rpcx stream (XDR data stream) module.
 * Copyright (c) 2002-2006 Oak Ridge National Laboratory.
 *
 * For more information see the following files in the source distribution top-
 * level directory or package data directory (usually /usr/local/share/package):
 *
 * - README    for general package information.
 * - INSTALL   for package install information.
 * - COPYING   for package license information and copying conditions.
 * - AUTHORS   for package authors information.
 * - ChangeLog for package changes information.
 *
 ******************************************************************************/

/******************************************************************************
 *
 * Macros
 *
 ******************************************************************************/

#ifndef ntohll
#if     __BYTE_ORDER == __BIG_ENDIAN
#define ntohll(value) value
#elif   __BYTE_ORDER == __LITTLE_ENDIAN
#define ntohll(value) __bswap64(value)
#endif  /* __BYTE_ORDER == __LITTLE_ENDIAN */
#endif  /* ntohll */

#ifndef htonll
#if     __BYTE_ORDER == __BIG_ENDIAN
#define htonll(value) value
#elif   __BYTE_ORDER == __LITTLE_ENDIAN
#define htonll(value) __bswap64(value)
#endif  /* __BYTE_ORDER == __LITTLE_ENDIAN */
#endif  /* htonll */

/******************************************************************************
 *
 * Includes
 *
 ******************************************************************************/

/* Main module header file. */
#include "rmix-rpcx.h"

/******************************************************************************
 *
 * Data Types
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Function Prototypes
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Data
 *
 ******************************************************************************/

/* None at this time. */

/******************************************************************************
 *
 * Functions
 *
 ******************************************************************************/

/*
 * Writes a call into an outgoing XDR data stream.
 *
 * ostream = The outgoing XDR data stream.
 * callid  = The call id.
 * prognum = The program number.
 * progver = The program version.
 * procnum = The procedure number.
 * creflav = The authentication credential flavor.
 * cresize = The authentication credential body length.
 * crebody = The authentication credential body.
 * verflav = The authentication verifier flavor.
 * versize = The authentication verifier body length.
 * verbody = The authentication verifier body.
 * sigstr  = The type signature.
 * inary   = The input values array.
 * incnt   = The input values array size.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_write_call"
int rmix_rpcx_stream_write_call (rmix_ostream_t               *ostream,
                                 unsigned int                  callid,
                                 unsigned int                  prognum,
                                 unsigned int                  progver,
                                 unsigned int                  procnum,
                                 rmix_rpcx_stream_authflav_t   creflav,
                                 unsigned int                  cresize,
                                 const char                   *crebody,
                                 rmix_rpcx_stream_authflav_t   verflav,
                                 unsigned int                  versize,
                                 const char                   *verbody,
                                 const char                   *sigstr,
                                 const void                  **inary,
                                 unsigned int                  incnt) {
  int          mtype;
  unsigned int rpcver;
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check crebody parameter. */
  if ((NULL == crebody)&&(0 != cresize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check cresize parameter. */
  if ((0 == cresize)&&(NULL != crebody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cresize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if ((NULL == verbody)&&(0 != versize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if ((0 == versize)&&(NULL != verbody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check inary parameter. */
  if ((NULL == inary)&&(0 != incnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(inary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check incnt parameter. */
  if ((0 == incnt)&&(NULL != inary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(incnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Write call id. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(callid), &callid)) {
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to write call id)))
      errno = errno2;
    }
    return -1;
  }
  /* Write message type. */
  mtype = RMIX_RPCX_STREAM_MSGTYPE_CALL;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(mtype), &mtype)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write message type)))
    errno = errno2;
    return -1;
  }
  /* Write RPC version. */
  rpcver = 2;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(rpcver), &rpcver)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write RPC version)))
    errno = errno2;
    return -1;
  }
  /* Write program number. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(prognum), &prognum)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write program number)))
    errno = errno2;
    return -1;
  }
  /* Write program version. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(progver), &progver)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write program version)))
    errno = errno2;
    return -1;
  }
  /* Write procedure number. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(procnum), &procnum)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write procedure number)))
    errno = errno2;
    return -1;
  }
  /* Write authentication credential flavor. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(creflav), &creflav)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication credential flavor)))
    errno = errno2;
    return -1;
  }
  /* Write authentication credential body length. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(cresize), &cresize)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication credential body length)))
    errno = errno2;
    return -1;
  }
  /* Write authentication credential body. */
  if (0 != rmix_rpcx_stream_write(ostream, cresize, sizeof(char), crebody)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication credential body)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier flavor. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(verflav), &verflav)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier flavor)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier body length. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(versize), &versize)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier body length)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier body. */
  if (0 != rmix_rpcx_stream_write(ostream, versize, sizeof(char), verbody)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier body)))
    errno = errno2;
    return -1;
  }
  /* Write procedure parameters. */
  if (0 != rmix_rpcx_stream_valary_write(ostream, inary, incnt, sigstr)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write procedure parameters)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Reads a call without parmeters from an incoming XDR data stream.
 * Writes error messages to an outgoing XDR data stream.
 *
 * istream = The incoming XDR data stream.
 * ostream = The outgoing XDR data stream.
 * callid  = The call id (return).
 * prognum = The program number (return).
 * progver = The program version (return).
 * procnum = The procedure number (return).
 * creflav = The authentication credential flavor (return).
 * cresize = The authentication credential body length (return).
 * crebody = The authentication credential body (return).
 * verflav = The authentication verifier flavor (return).
 * versize = The authentication verifier body length (return).
 * verbody = The authentication verifier body (return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_read_call"
int rmix_rpcx_stream_read_call (rmix_istream_t               *istream,
                                rmix_ostream_t               *ostream,
                                unsigned int                 *callid,
                                unsigned int                 *prognum,
                                unsigned int                 *progver,
                                unsigned int                 *procnum,
                                rmix_rpcx_stream_authflav_t  *creflav,
                                unsigned int                 *cresize,
                                char                        **crebody,
                                rmix_rpcx_stream_authflav_t  *verflav,
                                unsigned int                 *versize,
                                char                        **verbody) {
  int          mtype;
  unsigned int rpcver;
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check istream->close parameter. */
  if (NULL == istream->close) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream->close parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check istream->incoming parameter. */
  if (NULL == istream->incoming) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream->incoming parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check callid parameter. */
  if (NULL == callid) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(callid parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check prognum parameter. */
  if (NULL == prognum) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(prognum parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check progver parameter. */
  if (NULL == progver) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(progver parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check procnum parameter. */
  if (NULL == procnum) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(procnum parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check creflav parameter. */
  if (NULL == creflav) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(creflav parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check cresize parameter. */
  if (NULL == cresize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(cresize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check crebody parameter. */
  if (NULL == crebody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(crebody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verflav parameter. */
  if (NULL == verflav) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verflav parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if (NULL == versize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if (NULL == verbody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Read call id. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*callid), callid)) {
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to read call id)))
      errno = errno2;
    }
    return -1;
  }
  /* Read message type. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(mtype), &mtype)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read message type)))
    errno = errno2;
    return -1;
  }
  /* Check message type. */
  if (RMIX_RPCX_STREAM_MSGTYPE_CALL != mtype) {
    errno = EPROTO;
    RMIX_LOG((RMIX_WARN(wrong message type %d), mtype))
    errno = EPROTO;
    return -1;
  }
  /* Read RPC version. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(rpcver), &rpcver)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read RPC version)))
    errno = errno2;
    return -1;
  }
  /* Check RPC version. */
  if (2 != rpcver) {
    unsigned int low, high;
    rmix_rpcx_stream_replystat_t  replystat;
    rmix_rpcx_stream_rejectstat_t rejectstat;
    errno = EPROTO;
    RMIX_LOG((RMIX_WARN(wrong RPC version %d), rpcver))
    /* Close the incoming XDR data stream. */
    if (0 != istream->close(istream->incoming)) {
      RMIX_LOG((RMIX_WARN(unable to close the incoming XDR data stream)))
      errno = EPROTO;
      return -1;
    }
    /* Write call id. */
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(*callid), callid)) {
      RMIX_LOG((RMIX_WARN(unable to write call id)))
      errno = EPROTO;
      return -1;
    }
    /* Write message type. */
    mtype = RMIX_RPCX_STREAM_MSGTYPE_REPLY;
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(mtype), &mtype)) {
      RMIX_LOG((RMIX_WARN(unable to write message type)))
      errno = EPROTO;
      return -1;
    }
    /* Write reply stat. */
    replystat = RMIX_RPCX_STREAM_REPLYSTAT_DENIED;
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(replystat),
                                                      &replystat)) {
      RMIX_LOG((RMIX_WARN(unable to write reply stat)))
      errno = EPROTO;
      return -1;
    }
    /* Write reject stat. */
    rejectstat = RMIX_RPCX_STREAM_REJECTSTAT_RPC_MISMATCH;
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(rejectstat),
                                                      &rejectstat)) {
      RMIX_LOG((RMIX_WARN(unable to write reject stat)))
      errno = EPROTO;
      return -1;
    }
    /* Write lowest supported version. */
    low = 2;
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(low), &low)) {
      RMIX_LOG((RMIX_WARN(unable to write lowest supported version)))
      errno = EPROTO;
      return -1;
    }
    /* Write highest supported version. */
    high = 2;
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(high), &high)) {
      RMIX_LOG((RMIX_WARN(unable to write highest supported version)))
      errno = EPROTO;
      return -1;
    }
    errno = EPROTO;
    return -1;
  }
  /* Read program number. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*prognum), prognum)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read program number)))
    errno = errno2;
    return -1;
  }
  /* Read program version. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*progver), progver)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read program version)))
    errno = errno2;
    return -1;
  }
  /* Read procedure number. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*procnum), procnum)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read procedure number)))
    errno = errno2;
    return -1;
  }
  /* Read authentication credential flavor. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*creflav), creflav)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read authentication credential flavor)))
    errno = errno2;
    return -1;
  }
  /* Read authentication credential body length. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*cresize), cresize)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read authentication credential body length)))
    errno = errno2;
    return -1;
  }
  /* Check authentication credential body length. */
  if (0 != *cresize) {
    /* Allocate authentication credential body. */
    if (NULL == (*crebody = malloc(*cresize))) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to allocate authentication credential body)))
      errno = errno2;
      return -1;
    }
    /* Read authentication credential body. */
    if (0 != rmix_rpcx_stream_read(istream, *cresize, sizeof(char), *crebody)) {
      int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
      RMIX_LOG((RMIX_WARN(unable to read authentication credential body)))
      /* Deallocate authentication credential body. */
      if (NULL != *crebody) {
        free(*crebody);
        *crebody = NULL;
        *cresize = 0;
      }
      errno = errno2;
      return -1;
    }
  } else {
    /* Initialize authentication credential body. */
    *crebody = NULL;
  }
  /* Read authentication verifier flavor. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*verflav), verflav)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read authentication verifier flavor)))
    /* Deallocate authentication credential body. */
    if (NULL != *crebody) {
      free(*crebody);
      *crebody = NULL;
      *cresize = 0;
    }
    errno = errno2;
    return -1;
  }
  /* Read authentication verifier body length. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*versize), versize)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read authentication verifier body length)))
    /* Deallocate authentication credential body. */
    if (NULL != *crebody) {
      free(*crebody);
      *crebody = NULL;
      *cresize = 0;
    }
    errno = errno2;
    return -1;
  }
  /* Check authentication verifier body length. */
  if (0 != *versize) {
    /* Allocate authentication verifier body. */
    if (NULL == (*verbody = malloc(*versize))) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to allocate authentication verifier body)))
      /* Deallocate authentication credential body. */
      if (NULL != *crebody) {
        free(*crebody);
        *crebody = NULL;
        *cresize = 0;
      }
      errno = errno2;
      return -1;
    }
    /* Read authentication verifier body. */
    if (0 != rmix_rpcx_stream_read(istream, *versize, sizeof(char), *verbody)) {
      int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
      RMIX_LOG((RMIX_WARN(unable to read authentication verifier body)))
      /* Deallocate authentication verifier body. */
      if (NULL != *verbody) {
        free(*verbody);
        *verbody = NULL;
        *versize = 0;
      }
      /* Deallocate authentication credential body. */
      if (NULL != *crebody) {
        free(*crebody);
        *crebody = NULL;
        *cresize = 0;
      }
      errno = errno2;
      return -1;
    }
  } else {
    /* Initialize authentication verifier body. */
    *verbody = NULL;
  }
  return 0;
}

/*
 * Creates and reads call parameters from an incoming XDR data stream.
 * Writes error messages to an outgoing XDR data stream.
 *
 * istream = The incoming XDR data stream.
 * ostream = The outgoing XDR data stream.
 * sigstr  = The type signature.
 * inary   = The input values array (return).
 * incnt   = The input values array size (return).
 * callid  = The call id (for error messages).
 * verflav = The authentication verifier flavor (for error messages).
 * versize = The authentication verifier body length (for error messages).
 * verbody = The authentication verifier body (for error messages).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_read_call2"
int rmix_rpcx_stream_read_call2 (rmix_istream_t                *istream,
                                 rmix_ostream_t                *ostream,
                                 const char                    *sigstr,
                                 void                        ***inary,
                                 unsigned int                  *incnt,
                                 unsigned int                   callid,
                                 rmix_rpcx_stream_authflav_t    verflav,
                                 unsigned int                   versize,
                                 const char                    *verbody) {
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check inary parameter. */
  if (NULL == inary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(inary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check incnt parameter. */
  if (NULL == incnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(incnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if ((NULL == verbody)&&(0 != versize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if ((0 == versize)&&(NULL != verbody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Read procedure parameters. */
  if (0 != rmix_rpcx_stream_valary_read2(istream, inary, incnt, sigstr)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to read procedure parameters)))
    /* Close the incoming XDR data stream. */
    if (0 != istream->close(istream->incoming)) {
      RMIX_LOG((RMIX_WARN(unable to close the incoming XDR data stream)))
      errno = errno2;
      return -1;
    }
    /* Write garbled args error message. */
    if (0 != rmix_rpcx_stream_write_error(ostream, callid, verflav, versize,
               verbody, RMIX_RPCX_STREAM_ACCEPTSTAT_GARBAGE_ARGS, 0, 0)) {
      RMIX_LOG((RMIX_WARN(unable to write garbled args error message)))
      errno = errno2;
      return -1;
    }
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Writes a deny message with an authentication error into an outgoing XDR data
 * stream.
 *
 * ostream  = The outgoing XDR data stream.
 * callid   = The call id.
 * authstat = The authentication status.
 * return   = 0 on success or -1 on any error with errno set appropriately.
 *            ECONNABORTED = Connection closed before data transfer.
 *            EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_write_deny"
int rmix_rpcx_stream_write_deny (rmix_ostream_t              *ostream,
                                 unsigned int                 callid,
                                 rmix_rpcx_stream_authstat_t  authstat) {
  int                           mtype;
  rmix_rpcx_stream_replystat_t  replystat;
  rmix_rpcx_stream_rejectstat_t rejectstat;
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Write call id. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(callid), &callid)) {
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to write call id)))
      errno = errno2;
    }
    return -1;
  }
  /* Write message type. */
  mtype = RMIX_RPCX_STREAM_MSGTYPE_REPLY;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(mtype), &mtype)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write message type)))
    errno = errno2;
    return -1;
  }
  /* Write reply stat. */
  replystat = RMIX_RPCX_STREAM_REPLYSTAT_DENIED;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(replystat), &replystat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write reply stat)))
    errno = errno2;
    return -1;
  }
  /* Write reject stat. */
  rejectstat = RMIX_RPCX_STREAM_REJECTSTAT_AUTH_ERROR;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(rejectstat),
                                                    &rejectstat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write reject stat)))
    errno = errno2;
    return -1;
  }
  /* Write authentication stat. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(authstat), &authstat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication stat)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Writes an accept message with an error into an outgoing XDR data stream.
 *
 * ostream = The outgoing XDR data stream.
 * callid  = The call id.
 * verflav = The authentication verifier flavor.
 * versize = The authentication verifier body length.
 * verbody = The authentication verifier body.
 * acceptstat = The accept status.
 * low     = The lowest version supported (if program version mismatch).
 * high    = The highest version supported (if program version mismatch).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_write_error"
int rmix_rpcx_stream_write_error(rmix_ostream_t                *ostream,
                                 unsigned int                   callid,
                                 rmix_rpcx_stream_authflav_t    verflav,
                                 unsigned int                   versize,
                                 const char                    *verbody,
                                 rmix_rpcx_stream_acceptstat_t  accptstat,
                                 int                            low,
                                 int                            high) {
  int                          mtype;
  rmix_rpcx_stream_replystat_t replystat;
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if ((NULL == verbody)&&(0 != versize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if ((0 == versize)&&(NULL != verbody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Write call id. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(callid), &callid)) {
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to write call id)))
      errno = errno2;
    }
    return -1;
  }
  /* Write message type. */
  mtype = RMIX_RPCX_STREAM_MSGTYPE_REPLY;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(mtype), &mtype)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write message type)))
    errno = errno2;
    return -1;
  }
  /* Write reply stat. */
  replystat = RMIX_RPCX_STREAM_REPLYSTAT_ACCEPTED;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(replystat),
                                                       &replystat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write reply stat)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier flavor. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(verflav), &verflav)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier flavor)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier body length. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(versize), &versize)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier body length)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier body. */
  if (0 != rmix_rpcx_stream_write(ostream, versize,
                                     sizeof(char), verbody)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier body)))
    errno = errno2;
    return -1;
  }
  /* Write accept stat. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(accptstat), &accptstat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write accept stat)))
    errno = errno2;
    return -1;
  }
  /* Check accept stat. */
  if (accptstat == RMIX_RPCX_STREAM_ACCEPTSTAT_PROG_MISMATCH) {
    /* Write lowest supported version. */
    low = 2;
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(low), &low)) {
      int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
      RMIX_LOG((RMIX_WARN(unable to write lowest supported version)))
      errno = errno2;
      return -1;
    }
    /* Write highest supported version. */
    if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(high), &high)) {
      int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
      RMIX_LOG((RMIX_WARN(unable to write highest supported version)))
      errno = errno2;
      return -1;
    }
  }
  return 0;
}

/*
 * Writes a reply message w/o procedure return into an outgoing XDR data stream.
 *
 * ostream = The outgoing XDR data stream.
 * callid  = The call id.
 * verflav = The authentication verifier flavor.
 * versize = The authentication verifier body length.
 * verbody = The authentication verifier body.
 * sigstr  = The type signature.
 * outary  = The output values array (also return).
 * outcnt  = The output values array size (also return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_write_reply"
int rmix_rpcx_stream_write_reply(rmix_ostream_t              *ostream,
                                 unsigned int                 callid,
                                 rmix_rpcx_stream_authflav_t  verflav,
                                 unsigned int                 versize,
                                 const char                  *verbody) {
  int                           mtype;
  rmix_rpcx_stream_replystat_t  replystat;
  rmix_rpcx_stream_acceptstat_t acceptstat;
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if ((NULL == verbody)&&(0 != versize)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if ((0 == versize)&&(NULL != verbody)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Write call id. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(callid), &callid)) {
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to write call id)))
      errno = errno2;
    }
    return -1;
  }
  /* Write message type. */
  mtype = RMIX_RPCX_STREAM_MSGTYPE_REPLY;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(mtype), &mtype)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write message type)))
    errno = errno2;
    return -1;
  }
  /* Write reply stat. */
  replystat = RMIX_RPCX_STREAM_REPLYSTAT_ACCEPTED;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(replystat), &replystat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write reply stat)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier flavor. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(verflav), &verflav)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier flavor)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier body length. */
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(versize), &versize)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier body length)))
    errno = errno2;
    return -1;
  }
  /* Write authentication verifier body. */
  if (0 != rmix_rpcx_stream_write(ostream, versize,
                                  sizeof(char), verbody)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write authentication verifier body)))
    errno = errno2;
    return -1;
  }
  /* Write accept stat. */
  acceptstat = RMIX_RPCX_STREAM_ACCEPTSTAT_SUCCESS;
  if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(acceptstat),
                                                    &acceptstat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write accept stat)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Writes and destroys the procedure return of a reply message.
 *
 * ostream = The outgoing XDR data stream.
 * sigstr  = The type signature.
 * outary  = The output values array (also return).
 * outcnt  = The output values array size (also return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_write_reply2"
int rmix_rpcx_stream_write_reply2(rmix_ostream_t   *ostream,
                                  const char       *sigstr,
                                  void           ***outary,
                                  unsigned int     *outcnt) {
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check outary parameter. */
  if (NULL == outary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(outary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check outcnt parameter. */
  if (NULL == outcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(outcnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *outary parameter. */
  if ((NULL == *outary)&&(0 != *outcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*outary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *outcnt parameter. */
  if ((0 == *outcnt)&&(NULL != *outary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*outcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Write and destroy procedure return. */
  if (0 != rmix_rpcx_stream_valary_write2(ostream, outary, outcnt, sigstr)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to write and destroy procedure return)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Reads a reply message w/o procedure return from an incoming XDR data stream.
 *
 * istream = The incoming XDR data stream.
 * callid  = The call id.
 * verflav = The authentication verifier flavor (return).
 * versize = The authentication verifier body length (return).
 * verbody = The authentication verifier body (return).
 * low     = The lowest version supported (return if RPC/progver mismatch).
 * high    = The highest version supported (return if RPC/progver mismatch).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 *           ESRCH        = Remote prognum unavailable.
 *           ENOTSUP      = Remote progver mismatch.
 *           EOPNOTSUPP   = Remote procnum unavailable.
 *           EBADMSG      = Remote garbled args.
 *           ENOSYS       = Remote system error.
 *           ENOPROTOOPT  = Remote RPC version mismatch.
 *           EACCES       = Remote authentication error.
 *           EPROTO       = Protocol error.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_read_reply"
int rmix_rpcx_stream_read_reply (rmix_istream_t               *istream,
                                 unsigned int                  callid,
                                 rmix_rpcx_stream_authflav_t  *verflav,
                                 unsigned int                 *versize,
                                 char                        **verbody,
                                 int                          *low,
                                 int                          *high) {
  int                           mtype;
  unsigned int                  callid2;
  rmix_rpcx_stream_replystat_t  replystat;
  rmix_rpcx_stream_acceptstat_t acceptstat;
  rmix_rpcx_stream_rejectstat_t rejectstat;
  rmix_rpcx_stream_authstat_t   authstat;
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verflav parameter. */
  if (NULL == verflav) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verflav parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check versize parameter. */
  if (NULL == versize) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(versize parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check verbody parameter. */
  if (NULL == verbody) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(verbody parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Read call id. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(callid2), &callid2)) {
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      int errno2 = errno;
      RMIX_LOG((RMIX_WARN(unable to read call id)))
      errno = errno2;
    }
    return -1;
  }
  /* Check for wrong call id. */
  if (callid != callid2) {
    errno = EPROTO;
    RMIX_LOG((RMIX_WARN(wrong call id)))
    errno = EPROTO;
    return -1;
  }
  /* Read message type. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(mtype), &mtype)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read message type)))
    errno = errno2;
    return -1;
  }
  /* Check for wrong message type. */
  if (RMIX_RPCX_STREAM_MSGTYPE_REPLY != mtype) {
    errno = EPROTO;
    RMIX_LOG((RMIX_WARN(wrong message type)))
    errno = EPROTO;
    return -1;
  }
  /* Read reply stat. */
  if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(replystat), &replystat)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read reply stat)))
    errno = errno2;
    return -1;
  }
  /* Check reply stat. */
  switch (replystat) {
    /* Check for accept message. */
    case RMIX_RPCX_STREAM_REPLYSTAT_ACCEPTED: {
      /* Read authentication verifier flavor. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*verflav), verflav)) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((RMIX_WARN(unable to read authentication verifier flavor)))
        errno = errno2;
        return -1;
      }
      /* Read authentication verifier body length. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(*versize), versize)) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((
          RMIX_WARN(unable to read authentication verifier body length)))
        errno = errno2;
        return -1;
      }
      /* Check authentication verifier body length. */
      if (0 != *versize) {
        /* Allocate authentication verifier body. */
        if (NULL == (*verbody = malloc(*versize))) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to allocate authentication verifier body)))
          errno = errno2;
          return -1;
        }
        /* Read authentication verifier body. */
        if (0 != rmix_rpcx_stream_read(istream, *versize,
                                       sizeof(char), *verbody)) {
          int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
          RMIX_LOG((RMIX_WARN(unable to read authentication verifier body)))
          /* Deallocate authentication verifier body. */
          if (NULL != *verbody) {
            free(*verbody);
            *verbody = NULL;
            *versize = 0;
          }
          errno = errno2;
          return -1;
        }
      } else {
        /* Initialize authentication verifier body. */
        *verbody = NULL;
      }
      /* Read accept stat. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(acceptstat),
                                                       &acceptstat)) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((RMIX_WARN(unable to read accept stat)))
        /* Deallocate authentication verifier body. */
        if (NULL != *verbody) {
          free(*verbody);
          *verbody = NULL;
          *versize = 0;
        }
        errno = errno2;
        return -1;
      }
      /* Check accept stat. */
      switch (acceptstat) {
        /* Check for success. */
        case RMIX_RPCX_STREAM_ACCEPTSTAT_SUCCESS: {
          return 0;
        }
        /* Check for prognum unavailable. */
        case RMIX_RPCX_STREAM_ACCEPTSTAT_PROG_UNAVAIL: {
          errno = ESRCH;
          RMIX_LOG((RMIX_WARN(remote prognum unavailable)))
          errno = ESRCH;
          return -1;
        }
        /* Check for progver mismatch. */
        case RMIX_RPCX_STREAM_ACCEPTSTAT_PROG_MISMATCH: {
          errno = ENOTSUP;
          RMIX_LOG((RMIX_WARN(remote progver mismatch)))
          /* Read lowest supported version. */
          if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(low), &low)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read lowest supported version)))
            errno = errno2;
            return -1;
          }
          /* Read highest supported version. */
          if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(high), &high)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read highest supported version)))
            errno = errno2;
            return -1;
          }
          errno = ENOTSUP;
          return -1;
        }
        /* Check for procnum unavailable. */
        case RMIX_RPCX_STREAM_ACCEPTSTAT_PROC_UNAVAIL: {
          errno = EOPNOTSUPP;
          RMIX_LOG((RMIX_WARN(remote procnum unavailable)))
          errno = EOPNOTSUPP;
          return -1;
        }
        /* Check for garbled args. */
        case RMIX_RPCX_STREAM_ACCEPTSTAT_GARBAGE_ARGS: {
          errno = EBADMSG;
          RMIX_LOG((RMIX_WARN(remote garbled args)))
          errno = EBADMSG;
          return -1;
        }
        /* Check for system error. */
        case RMIX_RPCX_STREAM_ACCEPTSTAT_SYSTEM_ERR: {
          errno = ENOSYS;
          RMIX_LOG((RMIX_WARN(remote system error)))
          errno = ENOSYS;
          return -1;
        }
        /* Catch wrong accept stat. */
        default: {
          errno = EPROTO;
          RMIX_LOG((RMIX_WARN(wrong accept stat)))
          errno = EPROTO;
          return -1;
        }
      }
    }
    /* Check for deny message. */
    case RMIX_RPCX_STREAM_REPLYSTAT_DENIED: {
      /* Read reject stat. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(rejectstat),
                                                          &rejectstat)) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((RMIX_WARN(unable to read reject stat)))
        errno = errno2;
        return -1;
      }
      /* Check reject stat. */
      switch (rejectstat) {
        /* Check for RPC version mismatch. */
        case RMIX_RPCX_STREAM_REJECTSTAT_RPC_MISMATCH: {
          errno = ENOPROTOOPT;
          RMIX_LOG((RMIX_WARN(remote RPC version mismatch)))
          /* Read lowest supported version. */
          if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(low), &low)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read lowest supported version)))
            errno = errno2;
            return -1;
          }
          /* Read highest supported version. */
          if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(high), &high)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read highest supported version)))
            errno = errno2;
            return -1;
          }
          errno = ENOPROTOOPT;
          return -1;
        }
        /* Check for authentication error. */
        case RMIX_RPCX_STREAM_REJECTSTAT_AUTH_ERROR: {
          errno = EACCES;
          RMIX_LOG((RMIX_WARN(remote authentication error)))
          /* Read authentication stat. */
          if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(authstat),
                                                           &authstat)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read authentication stat)))
            errno = errno2;
            return -1;
          }
          /* Check authentication stat. */
          switch (authstat) {
            /* Check for bad credentials. */
            case RMIX_RPCX_STREAM_AUTHSTAT_BADCRED: {
              RMIX_LOG((RMIX_WARN(bad credentials)))
              break;
            }
            /* Check for rejected credentials. */
            case RMIX_RPCX_STREAM_AUTHSTAT_REJECTEDCRED: {
              RMIX_LOG((RMIX_WARN(rejected credentials)))
              break;
            }
            /* Check for bad verifier. */
            case RMIX_RPCX_STREAM_AUTHSTAT_BADVERF: {
              RMIX_LOG((RMIX_WARN(bad verifier)))
              break;
            }
            /* Check for rejected verifier. */
            case RMIX_RPCX_STREAM_AUTHSTAT_REJECTEDVERF: {
              RMIX_LOG((RMIX_WARN(rejected verifier)))
              break;
            }
            /* Check for weak authentication. */
            case RMIX_RPCX_STREAM_AUTHSTAT_TOOWEAK: {
              RMIX_LOG((RMIX_WARN(weak authentication)))
              break;
            }
            /* Check for invalid reponse verifier. */
            case RMIX_RPCX_STREAM_AUTHSTAT_INVALIDRESP: {
              RMIX_LOG((RMIX_WARN(invalid reponse verifier)))
              break;
            }
            /* Check for failed authentication. */
            case RMIX_RPCX_STREAM_AUTHSTAT_FAILED: {
              RMIX_LOG((RMIX_WARN(failed authentication)))
              break;
            }
            /* Catch wrong authentication stat. */
            default: {
              errno = EPROTO;
              RMIX_LOG((RMIX_WARN(wrong authentication stat)))
              errno = EPROTO;
              return -1;
            }
          }
          errno = EACCES;
          return -1;
        }
        /* Catch wrong reject stat. */
        default: {
          errno = EPROTO;
          RMIX_LOG((RMIX_WARN(wrong reject stat)))
          errno = EPROTO;
          return -1;
        }
      }
    }
    /* Catch wrong reply stat. */
    default: {
      errno = EPROTO;
      RMIX_LOG((RMIX_WARN(wrong reply stat)))
      errno = EPROTO;
      return -1;
    }
  }
}

/*
 * Reads the procedure return of a reply message. String and variable array
 * values are allocated.
 *
 * istream = The incoming XDR data stream.
 * sigstr  = The type signature.
 * outary  = The output values array (also return).
 * outcnt  = The output values array size.
 * low     = The lowest version supported (return if RPC/progver mismatch).
 * high    = The highest version supported (return if RPC/progver mismatch).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_read_reply2"
int rmix_rpcx_stream_read_reply2 (rmix_istream_t  *istream,
                                  const char      *sigstr,
                                  void           **outary,
                                  unsigned int     outcnt) {
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check outary parameter. */
  if ((NULL == outary)&&(0 != outcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(outary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check outcnt parameter. */
  if ((0 == outcnt)&&(NULL != outary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(outcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Read procedure return. */
  if (0 != rmix_rpcx_stream_valary_read(istream, outary,
                                        outcnt, sigstr)) {
    int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
    RMIX_LOG((RMIX_WARN(unable to read procedure return)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Writes a values array into an outgoing XDR data stream.
 *
 * ostream = The outgoing XDR data stream.
 * valary  = The values array.
 * valcnt  = The values array size.
 * sigstr  = The type signature.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_valary_write"
int rmix_rpcx_stream_valary_write (rmix_ostream_t  *ostream,
                                   const void     **valary,
                                   unsigned int     valcnt,
                                   const char      *sigstr) {
  unsigned int validx, sigidx;
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary parameter. */
  if ((NULL == valary)&&(0 != valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if ((0 == valcnt)&&(NULL != valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Write values. */
  for (validx  = 0, sigidx = 0; '\0' != sigstr[sigidx];) {
#ifdef DEBUG
    /* Check for values array size too small. */
    if (validx >= valcnt) {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(values array size too small)))
      errno = EINVAL;
      return -1;
    }
#endif /* DEBUG */
    /* Write value. */
    if (0 != rmix_rpcx_stream_value_write(ostream, valary,
                                          &validx, sigstr, &sigidx)) {
      /* Check for other errors than connection closed before transfer. */
      if ((ECONNABORTED != errno)||(0 != validx)) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((RMIX_WARN(unable to write value)))
        errno = errno2;
      }
      return -1;
    }
  }
  return 0;
}

/*
 * Writes a values array into an outgoing XDR data stream and destroys it.
 *
 * istream = The incoming XDR data stream.
 * valary  = The values array.
 * valcnt  = The values array size (return).
 * sigstr  = The type signature.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_valary_write2"
int rmix_rpcx_stream_valary_write2 (rmix_ostream_t   *ostream,
                                    void           ***valary,
                                    unsigned int     *valcnt,
                                    const char       *sigstr) {
  rmix_valary_cleanup_t cleanup;
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if (NULL == valcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valary parameter. */
  if ((NULL == *valary)&&(0 != *valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check *valcnt parameter. */
  if ((0 == *valcnt)&&(NULL != *valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(*valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Push deallocate values array to pthread cleanup stack. */
  cleanup.valary = *valary;
  cleanup.valcnt = *valcnt;
  cleanup.sigstr = sigstr;
  pthread_cleanup_push(rmix_valary_cleanup, &cleanup);
  /* Write values array. */
  if (0 != rmix_rpcx_stream_valary_write(ostream, (const void**)*valary,
                                         *valcnt, sigstr)) {
    int errno2 = errno;
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      RMIX_LOG((RMIX_WARN(unable to write values array)))
    }
    /* Destroy values array. */
    if (0 != rmix_valary_destroy(valary, valcnt, sigstr)) {
      RMIX_LOG((RMIX_WARN(unable to destroy values array)))
    }
    errno = errno2;
    return -1;
  }
  /* Pop deallocate values array from pthread cleanup stack. */
  pthread_cleanup_pop(0);
  /* Destroy values array. */
  if (0 != rmix_valary_create(valary, valcnt, sigstr)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to destroy values array)))
    errno = errno2;
    return -1;
  }
  return 0;
}

/*
 * Reads a values array from an incoming XDR data stream. String and variable
 * array values are allocated.
 *
 * istream = The incoming XDR data stream.
 * valary  = The values array.
 * valcnt  = The values array size.
 * sigstr  = The type signature.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_valary_read"
int rmix_rpcx_stream_valary_read (rmix_istream_t  *istream,
                                  void           **valary,
                                  unsigned int     valcnt,
                                  const char      *sigstr) {
  unsigned int          validx, sigidx;
  rmix_valary_cleanup_t cleanup;
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary parameter. */
  if ((NULL == valary)&&(0 != valcnt)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if ((0 == valcnt)&&(NULL != valary)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is zero)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Push deallocate variable length values to pthread cleanup stack. */
  cleanup.valary = valary;
  cleanup.valcnt = valcnt;
  cleanup.sigstr = sigstr;
  pthread_cleanup_push(rmix_valary_cleanup2, &cleanup);
  /* Read values. */
  for (validx = 0, sigidx = 0; '\0' != sigstr[sigidx];) {
#ifdef DEBUG
    /* Check for values array size too small. */
    if (validx >= valcnt) {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(values array size too small)))
      errno = EINVAL;
      return -1;
    }
#endif /* DEBUG */
    /* Read value. */
    if (0 != rmix_rpcx_stream_value_read(istream, valary,
                                         &validx, sigstr, &sigidx)) {
      /* Check for other errors than connection closed before transfer. */
      if ((ECONNABORTED != errno)||(0 != validx)) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((RMIX_WARN(unable to read value)))
        errno = errno2;
      }
      return -1;
    }
  }
  /* Pop deallocate variable length values from pthread cleanup stack. */
  pthread_cleanup_pop(0);
  return 0;
}

/*
 * Creates and reads a values array from an incoming XDR data stream.
 *
 * istream = The incoming XDR data stream.
 * valary  = The values array.
 * valcnt  = The values array size (return).
 * sigstr  = The type signature.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_valary_read2"
int rmix_rpcx_stream_valary_read2 (rmix_istream_t   *istream,
                                   void           ***valary,
                                   unsigned int     *valcnt,
                                   const char       *sigstr) {
  rmix_valary_cleanup_t cleanup;
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valcnt parameter. */
  if (NULL == valcnt) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valcnt parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Create values array. */
  if (0 != rmix_valary_create(valary, valcnt, sigstr)) {
    int errno2 = errno;
    RMIX_LOG((RMIX_WARN(unable to create values array)))
    errno = errno2;
    return -1;
  }
  /* Push deallocate values array to pthread cleanup stack. */
  cleanup.valary = *valary;
  cleanup.valcnt = *valcnt;
  cleanup.sigstr = sigstr;
  pthread_cleanup_push(rmix_valary_cleanup, &cleanup);
  /* Read values array. */
  if (0 != rmix_rpcx_stream_valary_read(istream, *valary, *valcnt, sigstr)) {
    int errno2 = errno;
    /* Check for other errors than connection closed before transfer. */
    if (ECONNABORTED != errno) {
      RMIX_LOG((RMIX_WARN(unable to read values array)))
    }
    /* Destroy values array. */
    if (0 != rmix_valary_destroy(valary, valcnt, sigstr)) {
      RMIX_LOG((RMIX_WARN(unable to destroy values array)))
    }
    errno = errno2;
    return -1;
  }
  /* Pop deallocate values array from pthread cleanup stack. */
  pthread_cleanup_pop(0);
  return 0;
}

/*
 * Writes a value into an outgoing XDR data stream and advances the values array
 * and signature indices.
 *
 * ostream = The outgoing XDR data stream.
 * valary  = The values array.
 * validx  = The values index (also return).
 * sigstr  = The type signature.
 * sigidx  = The type signature index (also return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_value_write"
int rmix_rpcx_stream_value_write (rmix_ostream_t  *ostream,
                                  const void     **valary,
                                  unsigned int    *validx,
                                  const char      *sigstr,
                                  unsigned int    *sigidx) {
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check validx parameter. */
  if (NULL == validx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(validx parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigidx parameter. */
  if (NULL == sigidx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigidx parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for type in signature. */
  switch (sigstr[*sigidx]) {
    /* Check for end of signature. */
    case '\0': {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(unexpected end of signature)))
      errno = EINVAL;
      return 0;
    }
    /* Check for char type. */
    case RMIX_SIGNATURE_CTYPE(char):
    case RMIX_SIGNATURE_CTYPE(signed_char):
    case RMIX_SIGNATURE_CTYPE(unsigned_char): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write char value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(char),
                                      valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write char value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for short type. */
    case RMIX_SIGNATURE_CTYPE(signed_short):
    case RMIX_SIGNATURE_CTYPE(unsigned_short): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write short value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(short),
                                      valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write short value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for int type. */
    case RMIX_SIGNATURE_CTYPE(signed_int):
    case RMIX_SIGNATURE_CTYPE(unsigned_int): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write int value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(int),
                                     valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write int value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write long value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(long),
                                      valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write long value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for long long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write long long value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(long long),
                                      valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write long long value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for float type. */
    case RMIX_SIGNATURE_CTYPE(float): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write float value. */
      if (0 != rmix_rpcx_stream_write(ostream, sizeof(float),
                                      sizeof(char), valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write float value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for double type. */
    case RMIX_SIGNATURE_CTYPE(double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write double value. */
      if (0 != rmix_rpcx_stream_write(ostream, sizeof(double),
                                      sizeof(char), valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write double value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for long double type. */
    case RMIX_SIGNATURE_CTYPE(long_double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Write long double value. */
      if (0 != rmix_rpcx_stream_write(ostream, sizeof(long double),
                                      sizeof(char), valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write long double value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for string type. */
    case RMIX_SIGNATURE_CTYPE(string): {
      unsigned int slength;
      /* Advance signature index. */
      (*sigidx)++;
      /* Calculate string length. */
      slength = (NULL == valary[*validx])?0:strlen(valary[*validx]) + 1;
      /* Write string length value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(int), &slength)) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write string length value)))
          errno = errno2;
        }
        return -1;
      }
      /* Write string value. */
      if (0 != rmix_rpcx_stream_write(ostream, slength, sizeof(char),
                                      valary[*validx])) {
        int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
        RMIX_LOG((RMIX_WARN(unable to write string value)))
        errno = errno2;
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for variable array type. */
    case RMIX_SIGNATURE_CTYPE(variable_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Get variable array length from values array. */
      length = *(unsigned int*)valary[*validx];
      /* Write variable array length value. */
      if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(int), &length)) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write variable array length value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return 0;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write char array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(char), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write char array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write short array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(short), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write short array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write int array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(int), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write int array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write long array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(long), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write long array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write long long array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(long long), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write long long array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write float array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length * sizeof(float),
                                          sizeof(char), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write float array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write double array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length * sizeof(double),
                                          sizeof(char), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write double array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write long double array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length * sizeof(long double),
                                          sizeof(char), valary[*validx])) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write long double array value)))
            errno = errno2;
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Write string array members. */
          for (index = 0; index < length; index++) {
            unsigned int slength;
            /* Calculate string length. */
            slength = (NULL == ((char**)valary[*validx])[index])?
                      0:strlen(((char**)valary[*validx])[index]) + 1;
            /* Write string length value. */
            if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(int),
                                            &slength)) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to write string length value)))
              errno = errno2;
              return -1;
            }
            /* Write string value. */
            if (0 != rmix_rpcx_stream_write(ostream, slength, sizeof(char),
                       ((char**)valary[*validx])[index])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to write string value)))
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          /* Check for variable array length value of 0. */
          if (0 == length) {
            void         **valary2;
            unsigned int   valcnt2, sigidx2;
            /* Initialize array/structure array. */
            valary2 = NULL;
            valcnt2 = 0;
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Create array/structure array member. */
            if (0 != rmix_value_create(&valary2, &valcnt2, sigstr, &sigidx2)) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to create array/structure array member)))
              /* Destroy array/structure array member. */
              if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                           &sigstr[*sigidx])) {
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array member)))
              }
              errno = errno2;
              return -1;
            }
            /* Destroy array/structure array member. */
            if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                         &sigstr[*sigidx])) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
              errno = errno2;
              return -1;
            }
            /* Advance signature index. */
            *sigidx = sigidx2;
          } else {
            unsigned int validx2, index, sigidx2;
            /* Write array/structure array members. */
            for (validx2 = 0, index = 0; index < length; index++) {
              /* Set array/structure array signature index. */
              sigidx2 = *sigidx;
              /* Write array/structure array member. */
              if (0 != rmix_rpcx_stream_value_write(ostream,
                         (const void**)valary[*validx], &validx2,
                         sigstr, &sigidx2)) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((
                  RMIX_WARN(unable to write array/structure array member)))
                errno = errno2;
                return -1;
              }
            }
            /* Advance signature index. */
            *sigidx = sigidx2;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for fixed array type. */
    case RMIX_SIGNATURE_CTYPE(fixed_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return 0;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write char array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write char array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write short array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(short), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write short array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write int array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(int), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write int array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write long array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(long), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write long array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write long long array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length,
                                          sizeof(long long), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write long long array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write float array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length * sizeof(float),
                                          sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write float array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write double array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length * sizeof(double),
                                          sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write double array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Write long double array value. */
          if (0 != rmix_rpcx_stream_write(ostream, length * sizeof(long double),
                                          sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to write long double array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Write string array members. */
          for (index = 0; index < length; index++) {
            unsigned int slength;
            /* Calculate string length. */
            slength = (NULL == ((char**)valary[*validx])[index])?
                      0:strlen(((char**)valary[*validx])[index]) + 1;
            /* Write string length value. */
            if (0 != rmix_rpcx_stream_write(ostream, 1, sizeof(int),
                                            &slength)) {
              /* Check for other errors than connection closed before xfer. */
              if ((ECONNABORTED != errno)||(0 != index)) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((RMIX_WARN(unable to write string length value)))
                errno = errno2;
              }
              return -1;
            }
            /* Write string value. */
            if (0 != rmix_rpcx_stream_write(ostream, slength, sizeof(char),
                       ((char**)valary[*validx])[index])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to write string value)))
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          unsigned int validx2, index, sigidx2;
          /* Write array/structure array members. */
          for (validx2 = 0, index = 0; index < length; index++) {
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Write array/structure array member. */
            if (0 != rmix_rpcx_stream_value_write(ostream,
                       (const void**)valary[*validx], &validx2,
                       sigstr, &sigidx2)) {
              /* Check for other errors than connection closed before xfer. */
              if ((ECONNABORTED != errno)||(0 != index)) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((
                  RMIX_WARN(unable to write array/structure array member)))
                errno = errno2;
              }
              return -1;
            }
          }
          /* Advance signature index. */
          *sigidx = sigidx2;
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for structure type. */
    case RMIX_SIGNATURE_CTYPE(structure_): {
      unsigned int length, validx2, index;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Write structure members. */
      for (validx2 = 0, index = 0; index < length; index++) {
        /* Write structure member. */
        if (0 != rmix_rpcx_stream_value_write(ostream,
                                              (const void**)valary[*validx],
                                              &validx2, sigstr, sigidx)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write structure member)))
            errno = errno2;
          }
          return -1;
        }
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Catch invalid type in signature. */
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(invalid type in signature)))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Reads a value from an incoming XDR data stream and advances the values array
 * and signature indices. String and variable array values are allocated.
 *
 * istream = The incoming XDR data stream.
 * valary  = The values array.
 * validx  = The values index (also return).
 * sigstr  = The type signature.
 * sigidx  = The type signature index (also return).
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_value_read"
int rmix_rpcx_stream_value_read (rmix_istream_t  *istream,
                                 void           **valary,
                                 unsigned int    *validx,
                                 const char      *sigstr,
                                 unsigned int    *sigidx) {
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check valary parameter. */
  if (NULL == valary) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(valary parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check validx parameter. */
  if (NULL == validx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(validx parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigstr parameter. */
  if (NULL == sigstr) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigstr parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check sigidx parameter. */
  if (NULL == sigidx) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(sigidx parameter is null)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check for type in signature. */
  switch (sigstr[*sigidx]) {
    /* Check for end of signature. */
    case '\0': {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(unexpected end of signature)))
      errno = EINVAL;
      return 0;
    }
    /* Check for char type. */
    case RMIX_SIGNATURE_CTYPE(char):
    case RMIX_SIGNATURE_CTYPE(signed_char):
    case RMIX_SIGNATURE_CTYPE(unsigned_char): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read char value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(char),
                                     valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read char value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for short type. */
    case RMIX_SIGNATURE_CTYPE(signed_short):
    case RMIX_SIGNATURE_CTYPE(unsigned_short): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read short value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(short),
                                     valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read short value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for int type. */
    case RMIX_SIGNATURE_CTYPE(signed_int):
    case RMIX_SIGNATURE_CTYPE(unsigned_int): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read int value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(int),
                                     valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read int value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read long value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(long),
                                     valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read long value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for long long type. */
    case RMIX_SIGNATURE_CTYPE(signed_long_long):
    case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read long long value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(long long),
                                     valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read long long value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for float type. */
    case RMIX_SIGNATURE_CTYPE(float): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read float value. */
      if (0 != rmix_rpcx_stream_read(istream, sizeof(float),
                                     sizeof(char), valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read float value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for double type. */
    case RMIX_SIGNATURE_CTYPE(double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read double value. */
      if (0 != rmix_rpcx_stream_read(istream, sizeof(double),
                                     sizeof(char), valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read double value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for long double type. */
    case RMIX_SIGNATURE_CTYPE(long_double): {
      /* Advance signature index. */
      (*sigidx)++;
      /* Read long double value. */
      if (0 != rmix_rpcx_stream_read(istream, sizeof(long double),
                                     sizeof(char), valary[*validx])) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read long double value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for string type. */
    case RMIX_SIGNATURE_CTYPE(string): {
      unsigned int slength;
      /* Advance signature index. */
      (*sigidx)++;
      /* Read string length value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(int), &slength)) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read string length value)))
          errno = errno2;
        }
        return -1;
      }
      /* Check for string length value of 0. */
      if (0 == slength) {
        /* Set value to NULL. */
        valary[*validx] = NULL;
      } else {
        /* Allocate string value. */
        if (NULL == (valary[*validx] = malloc(slength * sizeof(char)))) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to allocate string value)))
          errno = errno2;
          return -1;
        }
        /* Read string value. */
        if (0 != rmix_rpcx_stream_read(istream, slength, sizeof(char),
                                       valary[*validx])) {
          int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
          RMIX_LOG((RMIX_WARN(unable to read string value)))
          /* Deallocate string value. */
          free(valary[*validx]);
          errno = errno2;
          return -1;
        }
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Check for variable array type. */
    case RMIX_SIGNATURE_CTYPE(variable_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Read variable array length value. */
      if (0 != rmix_rpcx_stream_read(istream, 1, sizeof(int), &length)) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read variable array length value)))
          errno = errno2;
        }
        return -1;
      }
      /* Advance value index. */
      (*validx)++;
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return 0;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for char array length value of not 0. */
          if (0 != length) {
            /* Allocate char array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(char)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate char array value)))
              errno = errno2;
              return -1;
            }
            /* Read char array value. */
            if (0 != rmix_rpcx_stream_read(istream, length,
                                           sizeof(char), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read char array value)))
              /* Deallocate char array value. */
              free(valary[*validx]);
              valary[*validx] = NULL;
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for short array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate short array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(short)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate short array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read short array value. */
            if (0 != rmix_rpcx_stream_read(istream, length,
                                           sizeof(short), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read short array value)))
              /* Deallocate char array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for int array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate int array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(int)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate int array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read int array value. */
            if (0 != rmix_rpcx_stream_read(istream, length,
                                           sizeof(int), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read int array value)))
              /* Deallocate int array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for long array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate long array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(long)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate long array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read long array value. */
            if (0 != rmix_rpcx_stream_read(istream, length,
                                           sizeof(long), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read long array value)))
              /* Deallocate long array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for long long array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate long long array value. */
            if (NULL == (valary[*validx] = malloc(length *
                                                  sizeof(long long)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate long long array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read long long array value. */
            if (0 != rmix_rpcx_stream_read(istream, length,
                                           sizeof(long long),
                                           valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read long long array value)))
              /* Deallocate long long array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for float array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate float array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(float)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate float array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read float array value. */
            if (0 != rmix_rpcx_stream_read(istream, length * sizeof(float),
                                           sizeof(char), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read float array value)))
              /* Deallocate float array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for double array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate double array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(double)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate double array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read double array value. */
            if (0 != rmix_rpcx_stream_read(istream, length * sizeof(double),
                                           sizeof(char), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read double array value)))
              /* Deallocate double array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for long double array length value of 0. */
          if (0 == length) {
            /* Set value to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate long double array value. */
            if (NULL == (valary[*validx] = malloc(length *
                                                  sizeof(long double)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate long double array value)))
              /* Reset variable array length in values array. */
              *(unsigned int*)valary[(*validx) - 1] = 0;
              errno = errno2;
              return -1;
            }
            /* Read long double array value. */
            if (0 != rmix_rpcx_stream_read(istream, length *
                                           sizeof(long double),
                                           sizeof(char), valary[*validx])) {
              int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
              RMIX_LOG((RMIX_WARN(unable to read long double array value)))
              /* Deallocate long double array value. */
              free(valary[*validx]);
              errno = errno2;
              return -1;
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Check for string array length value of 0. */
          if (0 == length) {
            /* Set string to NULL. */
            valary[*validx] = NULL;
          } else {
            /* Allocate string array value. */
            if (NULL == (valary[*validx] = malloc(length * sizeof(char*)))) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to allocate string array value)))
              errno = errno2;
              return -1;
            }
            /* Read string array members. */
            for (index = 0; index < length; index++) {
              unsigned int slength;
              if (0 != rmix_rpcx_stream_read(istream, 1,
                                             sizeof(int), &slength)) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((RMIX_WARN(unable to read string length value)))
                /* Deallocate string array members. */
                while (0 < index--) {
                  /* Deallocate string array member. */
                  if (NULL != ((char**)valary[*validx])[index]) {
                    free(((char**)valary[*validx])[index]);
                  }
                }
                /* Deallocate string array value. */
                free(valary[*validx]);
                errno = errno2;
                return -1;
              }
              /* Check for string length value of 0. */
              if (0 == slength) {
                /* Set value to NULL. */
                ((char**)valary[*validx])[index] = NULL;
              } else {
                /* Allocate string value. */
                if (NULL == (((char**)valary[*validx])[index] =
                             malloc(slength * sizeof(char)))) {
                  int errno2 = errno;
                  RMIX_LOG((RMIX_WARN(unable to allocate string value)))
                  /* Deallocate string array members. */
                  while (0 < index--) {
                    /* Deallocate string array member. */
                    if (NULL != ((char**)valary[*validx])[index]) {
                      free(((char**)valary[*validx])[index]);
                    }
                  }
                  /* Deallocate string array value. */
                  free(valary[*validx]);
                  errno = errno2;
                  return -1;
                }
                /* Read string value. */
                if (0 != rmix_rpcx_stream_read(istream, slength,
                           sizeof(char), ((char**)valary[*validx])[index])) {
                  int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                  RMIX_LOG((RMIX_WARN(unable to read string value)))
                  /* Deallocate string array members. */
                  do {
                    /* Deallocate string array member. */
                    if (NULL != ((char**)valary[*validx])[index]) {
                      free(((char**)valary[*validx])[index]);
                    }
                  } while (0 < index--);
                  /* Deallocate string array value. */
                  free(valary[*validx]);
                  errno = errno2;
                  return -1;
                }
              }
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          /* Check for variable array length value of 0. */
          if (0 == length) {
            void         **valary2;
            unsigned int   valcnt2, sigidx2;
            /* Initialize array/structure array. */
            valary2 = NULL;
            valcnt2 = 0;
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Create array/structure array member. */
            if (0 != rmix_value_create(&valary2, &valcnt2, sigstr, &sigidx2)) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to create array/structure array member)))
              /* Destroy array/structure array member. */
              if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                           &sigstr[*sigidx])) {
                RMIX_LOG((
                  RMIX_WARN(unable to destroy array/structure array member)))
              }
              errno = errno2;
              return -1;
            }
            /* Destroy array/structure array member. */
            if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                         &sigstr[*sigidx])) {
              int errno2 = errno;
              RMIX_LOG((
                RMIX_WARN(unable to destroy array/structure array member)))
              errno = errno2;
              return -1;
            }
            /* Advance signature index. */
            *sigidx = sigidx2;
          } else {
            void         **valary2;
            unsigned int   valcnt2, validx2, index, sigidx2;
            /* Create array/structure array and members. */
            for (valary2 = NULL, valcnt2 = 0,
                 index = 0; index < length; index++) {
              /* Set array/structure array signature index. */
              sigidx2 = *sigidx;
              /* Create array/structure array member. */
              if (0 != rmix_value_create(&valary2, &valcnt2,
                                         sigstr, &sigidx2)) {
                int errno2 = errno;
                RMIX_LOG((
                  RMIX_WARN(unable to create array/structure array member)))
                /* Destroy array/structure array member. */
                if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                             &sigstr[*sigidx])) {
                  RMIX_LOG((
                    RMIX_WARN(unable to destroy array/structure array members)))
                }
                errno = errno2;
                return -1;
              }
            }
            /* Read array/structure array members. */
            for (validx2 = 0, index = 0; index < length; index++) {
              /* Set array/structure array signature index. */
              sigidx2 = *sigidx;
              /* Read array/structure array member. */
              if (0 != rmix_rpcx_stream_value_read(istream,
                         valary2, &validx2, sigstr, &sigidx2)) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((
                  RMIX_WARN(unable to read array/structure array member)))
                /* Destroy array/structure array member. */
                if (0 != rmix_valary_destroy(&valary2, &valcnt2,
                                             &sigstr[*sigidx])) {
                  RMIX_LOG((
                    RMIX_WARN(unable to destroy array/structure array members)))
                }
                errno = errno2;
                return -1;
              }
            }
            /* Save array/structure array value. */
            valary[*validx] = valary2;
            /* Advance signature index. */
            *sigidx = sigidx2;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      /* Set variable array length in values array. */
      *(unsigned int*)valary[(*validx) - 2] = length;
      break;
    }
    /* Check for fixed array type. */
    case RMIX_SIGNATURE_CTYPE(fixed_array_): {
      unsigned int length;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Check for array type in signature. */
      switch (sigstr[*sigidx]) {
        /* Check for missing array type. */
        case '\0': {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(missing array type signature)))
          errno = EINVAL;
          return 0;
        }
        /* Check for char array type. */
        case RMIX_SIGNATURE_CTYPE(char):
        case RMIX_SIGNATURE_CTYPE(signed_char):
        case RMIX_SIGNATURE_CTYPE(unsigned_char): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read char array value. */
          if (0 != rmix_rpcx_stream_read(istream, length,
                                         sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read char array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for short array type. */
        case RMIX_SIGNATURE_CTYPE(signed_short):
        case RMIX_SIGNATURE_CTYPE(unsigned_short): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read short array value. */
          if (0 != rmix_rpcx_stream_read(istream, length,
                                         sizeof(short), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read short array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for int array type. */
        case RMIX_SIGNATURE_CTYPE(signed_int):
        case RMIX_SIGNATURE_CTYPE(unsigned_int): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read int array value. */
          if (0 != rmix_rpcx_stream_read(istream, length,
                                         sizeof(int), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read int array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read long array value. */
          if (0 != rmix_rpcx_stream_read(istream, length,
                                         sizeof(long), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read long array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long long array type. */
        case RMIX_SIGNATURE_CTYPE(signed_long_long):
        case RMIX_SIGNATURE_CTYPE(unsigned_long_long): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read long long array value. */
          if (0 != rmix_rpcx_stream_read(istream, length,
                                         sizeof(long long), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read long long array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for float array type. */
        case RMIX_SIGNATURE_CTYPE(float): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read float array value. */
          if (0 != rmix_rpcx_stream_read(istream, length * sizeof(float),
                                         sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read float array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for double array type. */
        case RMIX_SIGNATURE_CTYPE(double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read double array value. */
          if (0 != rmix_rpcx_stream_read(istream, length * sizeof(double),
                                         sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read double array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for long double array type. */
        case RMIX_SIGNATURE_CTYPE(long_double): {
          /* Advance signature index. */
          (*sigidx)++;
          /* Read long double array value. */
          if (0 != rmix_rpcx_stream_read(istream, length * sizeof(long double),
                                         sizeof(char), valary[*validx])) {
            /* Check for other errors than connection closed before transfer. */
            if (ECONNABORTED != errno) {
              int errno2 = errno;
              RMIX_LOG((RMIX_WARN(unable to read long double array value)))
              errno = errno2;
            }
            return -1;
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for string array type. */
        case RMIX_SIGNATURE_CTYPE(string): {
          unsigned int index;
          /* Advance signature index. */
          (*sigidx)++;
          /* Read string array members. */
          for (index = 0; index < length; index++) {
            unsigned int slength;
            /* Read string length. */
            if (0 != rmix_rpcx_stream_read(istream, 1,
                                           sizeof(int), &slength)) {
              int errno2 = errno;
              /* Check for other errors than connection closed before xfer. */
              if ((ECONNABORTED != errno)||(0 != index)) {
                errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((RMIX_WARN(unable to read string length value)))
              }
              /* Deallocate string array members. */
              while (0 < index--) {
                /* Deallocate string array member. */
                if (NULL != ((char**)valary[*validx])[index]) {
                  free(((char**)valary[*validx])[index]);
                }
              }
              errno = errno2;
              return -1;
            }
            /* Check for string length value of 0. */
            if (0 == slength) {
              /* Set value to NULL. */
              ((char**)valary[*validx])[index] = NULL;
            } else {
              /* Allocate string value. */
              if (NULL == (((char**)valary[*validx])[index] =
                           malloc(slength * sizeof(char)))) {
                int errno2 = errno;
                RMIX_LOG((RMIX_WARN(unable to allocate string value)))
                /* Deallocate string array members. */
                while (0 < index--) {
                  /* Deallocate string array member. */
                  if (NULL != ((char**)valary[*validx])[index]) {
                    free(((char**)valary[*validx])[index]);
                  }
                }
                errno = errno2;
                return -1;
              }
              /* Read string value. */
              if (0 != rmix_rpcx_stream_read(istream, slength,
                         sizeof(char), ((char**)valary[*validx])[index])) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((RMIX_WARN(unable to read string value)))
                /* Deallocate string array members. */
                do {
                  /* Deallocate string array member. */
                  if (NULL != ((char**)valary[*validx])[index]) {
                    free(((char**)valary[*validx])[index]);
                  }
                } while (0 < index--);
                errno = errno2;
                return -1;
              }
            }
          }
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Check for array array and array structure types. */
        case RMIX_SIGNATURE_CTYPE(variable_array_):
        case RMIX_SIGNATURE_CTYPE(fixed_array_):
        case RMIX_SIGNATURE_CTYPE(structure_): {
          unsigned int validx2, index, sigidx2;
          /* Read array/structure array members. */
          for (validx2 = 0, index = 0; index < length; index++) {
            /* Set array/structure array signature index. */
            sigidx2 = *sigidx;
            /* Read array/structure array member. */
            if (0 != rmix_rpcx_stream_value_read(istream,
                       (void**)valary[*validx], &validx2,
                       sigstr, &sigidx2)) {
              /* Check for other errors than connection closed before xfer. */
              if ((ECONNABORTED != errno)||(0 != index)) {
                int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
                RMIX_LOG((
                  RMIX_WARN(unable to read array/structure array member)))
                errno = errno2;
              }
              return -1;
            }
          }
          /* Advance signature index. */
          *sigidx = sigidx2;
          /* Advance value index. */
          (*validx)++;
          break;
        }
        /* Catch invalid array type in signature. */
        default: {
          errno = EINVAL;
          RMIX_LOG((RMIX_WARN(invalid array type in signature)))
          errno = EINVAL;
          return -1;
        }
      }
      break;
    }
    /* Check for structure type. */
    case RMIX_SIGNATURE_CTYPE(structure_): {
      unsigned int length, validx2, index;
      /* Advance signature index. */
      (*sigidx)++;
      /* Check for length in signature. */
      if (( '0' > sigstr[*sigidx])||
          ( '9' < sigstr[*sigidx])) {
        errno = EINVAL;
        RMIX_LOG((RMIX_WARN(missing length in signature)))
        errno = EINVAL;
        return -1;
      }
      /* Get length from signature. */
      length = atoi(&sigstr[*sigidx]);
      /* Advance signature index. */
      while (( '0' <= sigstr[*sigidx]) &&
             ( '9' >= sigstr[*sigidx]) &&
             ('\0' != sigstr[*sigidx])) {
        (*sigidx)++;
      }
      /* Read structure members. */
      for (validx2 = 0, index = 0; index < length; index++) {
        /* Read structure member. */
        if (0 != rmix_rpcx_stream_value_read(istream,
                                             (void**)valary[*validx],
                                             &validx2, sigstr, sigidx)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read structure member)))
            errno = errno2;
          }
          return -1;
        }
      }
      /* Advance value index. */
      (*validx)++;
      break;
    }
    /* Catch invalid type in signature. */
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(invalid type in signature)))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Writes values into an outgoing XDR data stream.
 *
 * ostream = The outgoing XDR data stream.
 * count   = The number of values to write.
 * length  = The byte length of a single value.
 * values  = The values buffer to write from.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_write"
int rmix_rpcx_stream_write (rmix_ostream_t *ostream,
                            unsigned int    count,
                            unsigned int    length,
                            const void     *values) {
#ifdef DEBUG
  /* Check ostream parameter. */
  if (NULL == ostream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check ostream->write parameter. */
  if (NULL == ostream->write) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream->write parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check ostream->outgoing parameter. */
  if (NULL == ostream->outgoing) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(ostream->outgoing parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check values parameter. */
  if ((NULL == values)&&(0 != count)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(values parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check count parameter. */
  if ((0 == count)&&(NULL != values)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check single value byte length. */
  switch (length) {
    /* Check for char values. */
    case 1: {
      uint32_t padding;
      /* Write char values. */
      if (0 != ostream->write(ostream->outgoing, count, values)) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to write values)))
          errno = errno2;
        }
        return -1;
      }
      /* Calculate padding length. */
      count   = (4 - (count % 4)) % 4;
      padding = 0;
      if (0 != count ) {
        /* Write padding. */
        if (0 != ostream->write(ostream->outgoing, count, &padding)) {
          int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
          RMIX_LOG((RMIX_WARN(unable to write padding)))
          errno = errno2;
          return -1;
        }
      }
      break;
    }
    /* Check for short values. */
    case 2: {
      unsigned int index;
      /* Write short values. */
      for (index = 0; index < count; index++) {
        /* Convert short value. */
        uint32_t value = htonl((uint32_t)((uint16_t*)values)[index]);
        /* Write short value. */
        if (0 != ostream->write(ostream->outgoing, sizeof(value), &value)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write short value)))
            errno = errno2;
          }
          return -1;
        }
      }
      break;
    }
    /* Check for long values. */
    case 4: {
      unsigned int index;
      /* Write long values. */
      for (index = 0; index < count; index++) {
        /* Convert long value. */
        uint32_t value = htonl(((uint32_t*)values)[index]);
        /* Write long value. */
        if (0 != ostream->write(ostream->outgoing, sizeof(value), &value)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write long value)))
            errno = errno2;
          }
          return -1;
        }
      }
      break;
    }
    /* Check for long long values. */
    case 8: {
      unsigned int index;
      /* Write long long values. */
      for (index = 0; index < count; index++) {
        /* Convert long long value. */
        uint64_t value = htonll(((uint64_t*)values)[index]);
        /* Write long long value. */
        if (0 != ostream->write(ostream->outgoing, sizeof(value), &value)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to write long long value)))
            errno = errno2;
          }
          return -1;
        }
      }
      break;
    }
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(value byte size of %u not supported), length))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/*
 * Reads values from an incoming XDR data stream.
 *
 * istream = The incoming XDR data stream.
 * count   = The number of values to read.
 * length  = The byte length of a single value.
 * values  = The values buffer to read into.
 * return  = 0 on success or -1 on any error with errno set appropriately.
 *           ECONNABORTED = Connection closed before data transfer.
 *           EPIPE        = Connection closed during data transfer.
 */
#undef  __FUNC__
#define __FUNC__ "rmix_rpcx_stream_read"
int rmix_rpcx_stream_read (rmix_istream_t *istream,
                           unsigned int    count,
                           unsigned int    length,
                           void           *values) {
#ifdef DEBUG
  /* Check istream parameter. */
  if (NULL == istream) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check istream->read parameter. */
  if (NULL == istream->read) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream->read parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check istream->incoming parameter. */
  if (NULL == istream->incoming) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(istream->incoming parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check values parameter. */
  if ((NULL == values)&&(0 != count)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(values parameter is null)))
    errno = EINVAL;
    return -1;
  }
  /* Check count parameter. */
  if ((0 == count)&&(NULL != values)) {
    errno = EINVAL;
    RMIX_LOG((RMIX_WARN(count parameter is zero)))
    errno = EINVAL;
    return -1;
  }
#endif /* DEBUG */
  /* Check single value byte length. */
  switch (length) {
    /* Check for char values. */
    case 1: {
      uint32_t padding;
      /* Read char values. */
      if (0 != istream->read(istream->incoming, count, values)) {
        /* Check for other errors than connection closed before transfer. */
        if (ECONNABORTED != errno) {
          int errno2 = errno;
          RMIX_LOG((RMIX_WARN(unable to read values)))
          errno = errno2;
        }
        return -1;
      }
      /* Calculate padding length. */
      count = (4 - (count % 4)) % 4;
      /* Read padding. */
      if (0 != count) {
        if (0 != istream->read(istream->incoming, count, &padding)) {
          int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
          RMIX_LOG((RMIX_WARN(unable to read padding)))
          errno = errno2;
          return -1;
        }
      }
      break;
    }
    /* Check for short values. */
    case 2: {
      unsigned int index;
      /* Read short values. */
      for (index = 0; index < count; index++) {
        uint32_t value;
        /* Read short value. */
        if (0 != istream->read(istream->incoming, sizeof(value), &value)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read short value)))
            errno = errno2;
          }
          return -1;
        }
        /* Convert short value. */
        ((uint16_t*)values)[index] = (uint16_t)ntohl(value);
      }
      break;
    }
    /* Check for long values. */
    case 4: {
      unsigned int index;
      /* Read long values. */
      for (index = 0; index < count; index++) {
        uint32_t value;
        /* Read long value. */
        if (0 != istream->read(istream->incoming, sizeof(value), &value)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read long value)))
            errno = errno2;
          }
          return -1;
        }
        /* Convert long value. */
        ((uint32_t*)values)[index] = ntohl(value);
      }
      break;
    }
    /* Check for long long values. */
    case 8: {
      unsigned int index;
      /* Read long long values. */
      for (index = 0; index < count; index++) {
        uint64_t value;
        /* Read long long value. */
        if (0 != istream->read(istream->incoming, sizeof(value), &value)) {
          /* Check for other errors than connection closed before transfer. */
          if ((ECONNABORTED != errno)||(0 != index)) {
            int errno2 = (ECONNABORTED == errno)?(errno = EPIPE):errno;
            RMIX_LOG((RMIX_WARN(unable to read long long value)))
            errno = errno2;
          }
          return -1;
        }
        /* Convert long long value. */
        ((uint64_t*)values)[index] = ntohll(value);
      }
      break;
    }
    default: {
      errno = EINVAL;
      RMIX_LOG((RMIX_WARN(value byte size of %u not supported), length))
      errno = EINVAL;
      return -1;
    }
  }
  return 0;
}

/******************************************************************************
 *
 * END OF FILE
 *
 ******************************************************************************/
